import json


class Writer:
    def __init__(self, project):
        self.project = project

    def save_project(self, project):
        """Save data to file"""
        with open(project.file_path, 'r') as f:
            orig_data = json.load(f)
        for key in project.data:
            if key in ('cards', 'encounter_sets'):
                continue
            orig_data[key] = project.data[key]

        with open(project.file_path, 'w') as f:
            json.dump(project.data, f, indent=4)
        self.dirty = False

    def save_card(self, card):
        print('saving card of project', card.id)
        with open(self.project.file_path, 'r') as f:
            orig_data = json.load(f)
        index = None
        for key, value in enumerate(orig_data['cards']):
            if value['id'] == card.id:
                index = key
                break
        if index:
            orig_data['cards'][index] = card.data
        with open(self.project.file_path, 'w') as f:
            json.dump(orig_data, f, indent=4)
        self.project.set_dirty(card.id, False)

    def save_all(self):
        """Save data to file"""
        with open(self.project.file_path, 'w') as f:
            json.dump(self.project.data, f, indent=4)
        self.project.clear_dirty()

    def save_face(self):
        pass

    def save_encounter(self):
        pass


class TranslationWriter(Writer):
    def __init__(self, translation):
        self.translation = translation

    def save_project(self, project):
        """Save data to file"""
        with open(self.translation.file_path, 'r') as f:
            orig_data = json.load(f)
        orig_data["project_name"] = project.name
        orig_data['guides'] = project.data['guides']

        with open(self.translation.file_path, 'w') as f:
            json.dump(orig_data, f, indent=4)
        project.dirty = False

    def save_encounter_set(self, encounter_set):
        """Save data to file"""
        with open(self.translation.file_path, 'r') as f:
            orig_data = json.load(f)
        if not orig_data.get('encounter_sets'):
            orig_data["encounter_sets"] = {}

        orig_data['encounter_sets'][encounter_set.id] = {}
        orig_data['encounter_sets'][encounter_set.id]['name'] = encounter_set.name

        with open(self.translation.file_path, 'w') as f:
            json.dump(orig_data, f, indent=4)

    def save_card(self, card):
        with open(self.translation.file_path, 'r') as f:
            orig_data = json.load(f)
        if not orig_data.get('cards'):
            orig_data["cards"] = {}

        orig_data['cards'][card.id] = {}
        orig_data['cards'][card.id]['name'] = card.name

        for side in ('front', 'back'):
            orig_data['cards'][card.id][side] = {}
            for field in ('name', 'text', 'flavor_text'):
                if field in card.data[side]:
                    orig_data['cards'][card.id][side][field] = card.data[side][field]

        with open(self.translation.file_path, 'w') as f:
            json.dump(orig_data, f, indent=4)

    def save_all(self):
        """Save data to file"""
        project = self.translation.project
        for encounter_set in project.encounter_sets:
            if encounter_set.dirty:
                self.save_encounter_set(encounter_set)
        for card in project.get_all_cards():
            if card.dirty:
                self.save_card(card)
        self.save_project(project)



"""
Face editor factory and base/json editors for Shoggoth
"""
import json

from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QLabel
)

from shoggoth.ui.face_editor import FaceEditor
from shoggoth.ui.text_editor import ArkhamTextEdit
from shoggoth.ui.player_editors import AssetEditor, EventEditor, SkillEditor, CustomizableEditor
from shoggoth.ui.encounter_editors import EnemyEditor, TreacheryEditor, LocationEditor, LocationBackEditor
from shoggoth.ui.campaign_editors import (
    ActEditor, ActBackEditor, AgendaEditor, AgendaBackEditor, ChaosEditor, StoryEditor
)
from shoggoth.ui.investigator_editors import InvestigatorEditor, InvestigatorBackEditor
from shoggoth.i18n import tr


class BaseEditor(FaceEditor):
    """Basic editor with just type field"""

    def setup_ui(self):
        """Type is already added by base class"""
        self.main_layout.addStretch()


class JsonEditor(FaceEditor):
    """Raw JSON editor for unknown types or manual editing"""

    def setup_ui(self):
        # Add a header explaining this is raw JSON mode
        header = QLabel(tr("JSON_EDITOR_HEADER"))
        header.setStyleSheet("font-size: 14pt; padding: 5px;")
        self.main_layout.addWidget(header)

        info = QLabel(tr("JSON_EDITOR_INFO"))
        info.setWordWrap(True)
        info.setStyleSheet("color: #666; padding: 5px;")
        self.main_layout.addWidget(info)

        # Create the JSON editor with syntax highlighting
        self.json_editor = ArkhamTextEdit()
        self.json_editor.setPlaceholderText("{\n  \"type\": \"asset\",\n  \"name\": \"Card Name\"\n}")

        # Enable JSON syntax highlighting
        try:
            from pygments.lexers import JsonLexer
            self.json_editor.set_lexer(JsonLexer())
        except:
            pass  # Syntax highlighting optional

        self.json_editor.textChanged.connect(self.on_json_changed)
        self.main_layout.addWidget(self.json_editor)

        # Validation status
        self.status_label = QLabel("")
        self.status_label.setStyleSheet("padding: 5px;")
        self.main_layout.addWidget(self.status_label)

        # Format button
        button_row = QHBoxLayout()

        format_btn = QPushButton(tr("BTN_FORMAT_JSON"))
        format_btn.clicked.connect(self.format_json)
        button_row.addWidget(format_btn)

        validate_btn = QPushButton(tr("BTN_VALIDATE"))
        validate_btn.clicked.connect(self.validate_json)
        button_row.addWidget(validate_btn)

        button_row.addStretch()

        button_widget = QWidget()
        button_widget.setLayout(button_row)
        self.main_layout.addWidget(button_widget)

    def format_json(self):
        """Format the JSON text"""
        try:
            text = self.json_editor.toPlainText()
            if text.strip():
                data = json.loads(text)
                formatted = json.dumps(data, indent=2)
                self.json_editor.setPlainText(formatted)
                self.status_label.setText(tr("STATUS_FORMATTED"))
                self.status_label.setStyleSheet("color: green; padding: 5px;")
        except json.JSONDecodeError as e:
            self.status_label.setText(tr("STATUS_JSON_INVALID").format(error=e))
            self.status_label.setStyleSheet("color: red; padding: 5px;")

    def validate_json(self):
        """Validate the JSON"""
        try:
            text = self.json_editor.toPlainText()
            if text.strip():
                json.loads(text)
                self.status_label.setText(tr("STATUS_VALID_JSON"))
                self.status_label.setStyleSheet("color: green; padding: 5px;")
            else:
                self.status_label.setText(tr("STATUS_EMPTY"))
                self.status_label.setStyleSheet("color: orange; padding: 5px;")
        except json.JSONDecodeError as e:
            self.status_label.setText(tr("STATUS_JSON_INVALID").format(error=e))
            self.status_label.setStyleSheet("color: red; padding: 5px;")

    def load_data(self):
        """Load face data as JSON"""
        super().load_data()  # load type field
        self.updating = True
        try:
            json_text = json.dumps(self.face.data, indent=2)
            self.json_editor.setPlainText(json_text)
            self.status_label.setText(tr("STATUS_JSON_LOADED"))
            self.status_label.setStyleSheet("color: green; padding: 5px;")
        except Exception as e:
            self.json_editor.setPlainText(tr("STATUS_ERROR_LOADING_JSON").format(error=e))
            self.status_label.setText(tr("STATUS_LOAD_ERROR").format(error=e))
            self.status_label.setStyleSheet("color: red; padding: 5px;")
        self.updating = False

    def on_json_changed(self):
        """Save JSON back to face"""
        if self.updating:
            return

        try:
            text = self.json_editor.toPlainText()
            if text.strip():
                data = json.loads(text)
                # Update face data
                self.face.data = data
                # Emit type_changed if type changed (to potentially switch editor)
                if 'type' in data and data['type'] != self.face.get('type'):
                    self.type_changed.emit()
                self.status_label.setText(tr("STATUS_SAVED"))
                self.status_label.setStyleSheet("color: green; padding: 5px;")
        except json.JSONDecodeError as e:
            self.status_label.setText(tr("STATUS_JSON_INVALID").format(error=str(e)[:50]))
            self.status_label.setStyleSheet("color: red; padding: 5px;")


# Mapping of editor types to classes
EDITOR_MAPPING = {
    'player': BaseEditor,
    'customizable_back': BaseEditor,
    'encounter': BaseEditor,
    'asset': AssetEditor,
    'event': EventEditor,
    'skill': SkillEditor,
    'investigator': InvestigatorEditor,
    'investigator_back': InvestigatorBackEditor,
    'location': LocationEditor,
    'location_back': LocationBackEditor,
    'treachery': TreacheryEditor,
    'enemy': EnemyEditor,
    'act': ActEditor,
    'act_back': ActBackEditor,
    'agenda': AgendaEditor,
    'agenda_back': AgendaBackEditor,
    'chaos': ChaosEditor,
    'customizable': CustomizableEditor,
    'story': StoryEditor,
}


def get_editor_for_face(face, parent=None):
    """Get the appropriate editor for a face"""
    editor_type = face.get_editor()
    editor_class = EDITOR_MAPPING.get(editor_type, JsonEditor)
    return editor_class(face, parent)



"""
Card-specific widgets (icons, illustration) for Shoggoth using PySide6
"""
from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QLabel, QPushButton
)
from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QPixmap

from shoggoth.ui.field_widgets import LabeledLineEdit
from shoggoth.files import overlay_dir
from shoggoth.i18n import tr


class IconsWidget(QWidget):
    """Widget for skill icons with +/- buttons for each icon type (WICAQ)"""

    # Icon order and their full names
    ICONS = [
        ('W', 'Willpower'),
        ('I', 'Intellect'),
        ('C', 'Combat'),
        ('A', 'Agility'),
        ('Q', 'Wild'),
    ]
    MAX_COUNT = 8

    iconsChanged = Signal(str)

    def __init__(self):
        super().__init__()
        self.counts = {letter: 0 for letter, _ in self.ICONS}
        self.labels = {}
        self._updating = False

        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(2)

        # Label
        icons_label = QLabel(tr("LABEL_ICONS"))
        layout.addWidget(icons_label)

        # Create one row per icon
        for letter, name in self.ICONS:
            row = QHBoxLayout()
            row.setSpacing(4)

            # Icon image
            icon_label = QLabel()
            icon_label.setToolTip(name)
            icon_path = overlay_dir / 'svg' / f"skill_icon_{letter}.svg"
            if icon_path.exists():
                pixmap = QPixmap(str(icon_path)).scaled(
                    20, 20, Qt.KeepAspectRatio, Qt.SmoothTransformation
                )
                icon_label.setPixmap(pixmap)
            else:
                icon_label.setText(letter)
            icon_label.setFixedSize(20, 20)
            row.addWidget(icon_label)

            # Minus button
            minus_btn = QPushButton("-")
            minus_btn.setFixedSize(24, 24)
            minus_btn.clicked.connect(lambda checked, l=letter: self.decrement(l))
            row.addWidget(minus_btn)

            # Count label
            count_label = QLabel("0")
            count_label.setAlignment(Qt.AlignCenter)
            count_label.setFixedWidth(20)
            self.labels[letter] = count_label
            row.addWidget(count_label)

            # Plus button
            plus_btn = QPushButton("+")
            plus_btn.setFixedSize(24, 24)
            plus_btn.clicked.connect(lambda checked, l=letter: self.increment(l))
            row.addWidget(plus_btn)

            # Name label
            name_label = QLabel(name)
            row.addWidget(name_label)

            row.addStretch()
            layout.addLayout(row)

        self.setLayout(layout)

    def increment(self, letter):
        """Increment count for an icon"""
        if self.counts[letter] < self.MAX_COUNT:
            self.counts[letter] += 1
            self.labels[letter].setText(str(self.counts[letter]))
            if not self._updating:
                self.iconsChanged.emit(self.get_icons_string())

    def decrement(self, letter):
        """Decrement count for an icon"""
        if self.counts[letter] > 0:
            self.counts[letter] -= 1
            self.labels[letter].setText(str(self.counts[letter]))
            if not self._updating:
                self.iconsChanged.emit(self.get_icons_string())

    def get_icons_string(self):
        """Get the icons as an ordered string like 'WWICQQ'"""
        result = ''
        for letter, _ in self.ICONS:
            result += letter * self.counts[letter]
        return result

    def set_icons_string(self, icons_str):
        """Set icons from a string like 'WWICQQ'"""
        self._updating = True
        # Reset all counts
        for letter, _ in self.ICONS:
            self.counts[letter] = 0

        # Count occurrences of each letter
        if icons_str:
            for char in icons_str.upper():
                if char in self.counts:
                    self.counts[char] = min(self.counts[char] + 1, self.MAX_COUNT)

        # Update labels
        for letter, _ in self.ICONS:
            self.labels[letter].setText(str(self.counts[letter]))

        self._updating = False


class IllustrationWidget(QWidget):
    """Widget for illustration settings"""

    # Signal emitted when illustration mode is toggled (enabled, face_side)
    illustration_mode_changed = Signal(bool, str)

    def __init__(self, face_side='front'):
        super().__init__()
        self.face_side = face_side
        self.illustration_mode = False
        layout = QVBoxLayout()

        # Image path
        path_layout = QHBoxLayout()
        self.path_input = LabeledLineEdit(tr("FIELD_IMAGE_PATH"))
        path_layout.addWidget(self.path_input)
        browse_btn = QPushButton(tr("BTN_BROWSE"))
        browse_btn.clicked.connect(self.browse_image)
        path_layout.addWidget(browse_btn)
        layout.addLayout(path_layout)

        # Pan and scale with edit button
        pan_scale_layout = QHBoxLayout()
        self.pan_y_input = LabeledLineEdit(tr("FIELD_PAN_Y"))
        self.pan_x_input = LabeledLineEdit(tr("FIELD_PAN_X"))
        self.scale_input = LabeledLineEdit(tr("FIELD_SCALE"))
        pan_scale_layout.addWidget(self.pan_y_input)
        pan_scale_layout.addWidget(self.pan_x_input)
        pan_scale_layout.addWidget(self.scale_input)

        # Edit position button
        self.edit_position_btn = QPushButton(tr("BTN_EDIT_POSITION"))
        self.edit_position_btn.setCheckable(True)
        self.edit_position_btn.setToolTip(tr("TOOLTIP_DRAG_PREVIEW"))
        self.edit_position_btn.clicked.connect(self.toggle_illustration_mode)
        pan_scale_layout.addWidget(self.edit_position_btn)

        layout.addLayout(pan_scale_layout)

        # Artist
        self.artist_input = LabeledLineEdit(tr("FIELD_ARTIST"))
        layout.addWidget(self.artist_input)

        self.setLayout(layout)

    def toggle_illustration_mode(self, checked):
        """Toggle illustration positioning mode"""
        self.illustration_mode = checked
        if checked:
            self.edit_position_btn.setText(tr("BTN_DONE"))
            self.edit_position_btn.setStyleSheet("background-color: #4a9eff; color: white;")
        else:
            self.edit_position_btn.setText(tr("BTN_EDIT_POSITION"))
            self.edit_position_btn.setStyleSheet("")
        self.illustration_mode_changed.emit(checked, self.face_side)

    def set_illustration_mode(self, enabled):
        """Set illustration mode programmatically"""
        self.edit_position_btn.setChecked(enabled)
        self.toggle_illustration_mode(enabled)

    def update_pan(self, delta_x, delta_y):
        """Update pan values from external input (e.g., preview drag)"""
        try:
            current_x = int(self.pan_x_input.text() or 0)
            current_y = int(self.pan_y_input.text() or 0)
            self.pan_x_input.setText(str(current_x + delta_x))
            self.pan_y_input.setText(str(current_y + delta_y))
        except ValueError:
            pass

    def update_scale(self, delta):
        """Update scale value from external input (e.g., preview scroll)"""
        try:
            current_scale = float(self.scale_input.text() or 1.0)
            new_scale = max(0.1, current_scale + delta)
            self.scale_input.setText(f"{new_scale:.3f}")
        except ValueError:
            pass

    def browse_image(self):
        """Browse for image file"""
        from PySide6.QtWidgets import QFileDialog
        from pathlib import Path

        file_path, _ = QFileDialog.getOpenFileName(
            self,
            tr("DLG_SELECT_IMAGE"),
            str(Path.home()),
            tr("FILTER_IMAGES")
        )
        if file_path:
            self.path_input.setText(file_path)



"""
Encounter Set Editor widget with thumbnails
"""
from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QGridLayout, QLabel,
    QPushButton, QScrollArea, QFrame, QMessageBox
)
from PySide6.QtCore import Qt, Signal, QSize, QThread
from PySide6.QtGui import QPixmap, QImage, QCursor
import threading
from io import BytesIO
from shoggoth.i18n import tr
from shoggoth.ui.field_widgets import LabeledLineEdit, FieldWidget


class ThumbnailWidget(QFrame):
    """A clickable thumbnail widget for a card"""

    clicked = Signal(str)  # Emits card ID

    def __init__(self, card_id, card_name):
        super().__init__()
        self.card_id = card_id
        self.card_name = card_name

        # Setup frame
        self.setFrameStyle(QFrame.Box | QFrame.Raised)
        self.setLineWidth(1)
        self.setCursor(QCursor(Qt.PointingHandCursor))

        # Layout
        layout = QVBoxLayout()
        layout.setContentsMargins(5, 5, 5, 5)

        # Image label - don't use setScaledContents, we'll handle aspect ratio manually
        self.image_label = QLabel()
        self.image_label.setAlignment(Qt.AlignCenter)
        self.image_label.setMinimumSize(200, 280)
        self.image_label.setText(tr("LOADING"))
        layout.addWidget(self.image_label)

        # Name label
        name_label = QLabel(card_name)
        name_label.setAlignment(Qt.AlignCenter)
        name_label.setWordWrap(True)
        name_label.setMaximumWidth(200)
        layout.addWidget(name_label)

        self.setLayout(layout)

    def set_image(self, image_buffer):
        """Set the thumbnail image from a BytesIO buffer with proper aspect ratio"""
        if not image_buffer:
            self.image_label.setText(tr("DLG_ERROR"))
            return

        try:
            image_buffer.seek(0)
            data = image_buffer.read()
            image = QImage.fromData(data)
            pixmap = QPixmap.fromImage(image)

            # Scale pixmap to fit in the label while maintaining aspect ratio
            scaled_pixmap = pixmap.scaled(
                200, 280,
                Qt.KeepAspectRatio,
                Qt.SmoothTransformation
            )
            self.image_label.setPixmap(scaled_pixmap)

            # Adjust frame size based on actual image orientation
            if pixmap.width() > pixmap.height():
                # Horizontal card
                self.image_label.setMinimumSize(280, 200)
                self.setMinimumWidth(290)
            else:
                # Vertical card
                self.image_label.setMinimumSize(200, 280)
                self.setMinimumWidth(210)

        except Exception as e:
            print(f"Error setting thumbnail: {e}")
            self.image_label.setText(tr("MSG_ERROR"))

    def mousePressEvent(self, event):
        """Handle mouse click"""
        if event.button() == Qt.LeftButton:
            self.clicked.emit(self.card_id)
        super().mousePressEvent(event)


class ThumbnailGenerator(QThread):
    """Thread for generating thumbnails in the background"""

    thumbnail_ready = Signal(str, object)  # card_id, image_buffer

    def __init__(self, cards, renderer):
        super().__init__()
        self.cards = cards
        self.renderer = renderer
        self._stop = False

    def run(self):
        """Generate thumbnails for all cards"""
        for card in self.cards:
            if self._stop:
                break

            try:
                # Generate thumbnail
                thumbnail = self.renderer.get_thumbnail(card)
                # Only emit if we haven't been stopped
                if not self._stop:
                    self.thumbnail_ready.emit(card.id, thumbnail)
            except Exception as e:
                print(f"Error generating thumbnail for {card.name}: {e}")

    def stop(self):
        """Stop the thumbnail generation"""
        self._stop = True
        # Don't wait for thread to finish here - parent will call wait() if needed


class EncounterSetEditor(QWidget):
    """Editor for encounter set with thumbnails"""

    card_clicked = Signal(object)  # Emits card object when thumbnail is clicked

    def __init__(self, encounter_set, renderer):
        super().__init__()
        self.encounter_set = encounter_set
        self.renderer = renderer
        self.thumbnail_widgets = {}
        self.thumbnail_thread = None

        self.setup_ui()
        self.load_data()
        self.generate_thumbnails()

    def setup_ui(self):
        """Setup the UI"""
        main_layout = QVBoxLayout()

        # Header section
        header_layout = QVBoxLayout()

        # Title
        title = QLabel(tr("TITLE_ENCOUNTER_SET").format(name=self.encounter_set.name))
        title.setStyleSheet("font-size: 20pt; font-weight: bold;")
        header_layout.addWidget(title)

        # Basic info form
        self.name_input = LabeledLineEdit(tr("FIELD_NAME"))
        header_layout.addWidget(self.name_input)

        self.code_input = LabeledLineEdit(tr("FIELD_CODE"))
        header_layout.addWidget(self.code_input)

        self.order_input = LabeledLineEdit(tr("FIELD_ORDER"))
        header_layout.addWidget(self.order_input)

        # Icon selection
        icon_layout = QHBoxLayout()
        icon_layout.addWidget(QLabel(tr("FIELD_ICON")))
        self.icon_input = LabeledLineEdit("")
        icon_layout.addWidget(self.icon_input)
        browse_btn = QPushButton(tr("BTN_BROWSE"))
        browse_btn.clicked.connect(self.browse_icon)
        icon_layout.addWidget(browse_btn)
        header_layout.addLayout(icon_layout)

        # Stats
        stats_label = QLabel()
        stats_label.setStyleSheet("color: #666; font-style: italic;")
        cards_count = len(self.encounter_set.cards)
        total_amount = sum(card.amount for card in self.encounter_set.cards)
        stats_label.setText(tr("STATS_CARDS_IN_SET").format(unique=cards_count, total=total_amount))
        header_layout.addWidget(stats_label)

        main_layout.addLayout(header_layout)

        # Separator
        separator = QFrame()
        separator.setFrameShape(QFrame.HLine)
        separator.setFrameShadow(QFrame.Sunken)
        main_layout.addWidget(separator)

        # Cards section label
        cards_label = QLabel(tr("LABEL_CARDS_IN_SET"))
        cards_label.setStyleSheet("font-size: 14pt; font-weight: bold; margin-top: 10px;")
        main_layout.addWidget(cards_label)

        # Scroll area for thumbnails
        scroll_area = QScrollArea()
        scroll_area.setWidgetResizable(True)
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        # Thumbnail grid container
        self.thumbnail_container = QWidget()
        self.thumbnail_grid = QGridLayout()
        self.thumbnail_grid.setSpacing(10)
        self.thumbnail_grid.setAlignment(Qt.AlignLeft | Qt.AlignTop)
        self.thumbnail_container.setLayout(self.thumbnail_grid)

        scroll_area.setWidget(self.thumbnail_container)
        main_layout.addWidget(scroll_area)

        # Action buttons
        button_layout = QHBoxLayout()

        new_card_btn = QPushButton(tr("BTN_NEW_CARD"))
        new_card_btn.clicked.connect(self.add_new_card)
        button_layout.addWidget(new_card_btn)

        button_layout.addStretch()

        delete_btn = QPushButton(tr("BTN_DELETE_ENCOUNTER_SET"))
        delete_btn.setStyleSheet("background-color: #d32f2f; color: white;")
        delete_btn.clicked.connect(self.delete_encounter_set)
        button_layout.addWidget(delete_btn)

        main_layout.addLayout(button_layout)

        self.setLayout(main_layout)

        # Setup field bindings
        self.setup_fields()

    def setup_fields(self):
        """Setup field bindings"""
        self.fields = [
            FieldWidget(self.name_input.input, 'name'),
            FieldWidget(self.code_input.input, 'code'),
            FieldWidget(self.order_input.input, 'order', int, str),
            FieldWidget(self.icon_input.input, 'icon'),
        ]

        # Connect signals
        for field in self.fields:
            widget = field.widget
            widget.textChanged.connect(lambda v, f=field: self.on_field_changed(f, v))

    def load_data(self):
        """Load encounter set data into fields"""
        for field in self.fields:
            field.update_from_card(self.encounter_set)

    def on_field_changed(self, field, value):
        """Handle field changes"""
        field.update_card(self.encounter_set, value)

    def browse_icon(self):
        """Browse for icon file"""
        from PySide6.QtWidgets import QFileDialog
        from pathlib import Path

        file_path, _ = QFileDialog.getOpenFileName(
            self,
            tr("DLG_SELECT_ICON"),
            str(Path.home()),
            tr("FILTER_IMAGES")
        )
        if file_path:
            self.icon_input.input.setText(file_path)

    def generate_thumbnails(self):
        """Generate thumbnails for all cards in the set"""
        cards = self.encounter_set.cards
        if not cards:
            return

        # Create thumbnail widgets
        col = 0
        row = 0
        cols_per_row = 3  # 3 thumbnails per row

        for card in cards:
            # Create thumbnail widget
            thumbnail = ThumbnailWidget(card.id, card.name)
            thumbnail.clicked.connect(lambda card_id, c=card: self.on_thumbnail_clicked(c))

            # Add to grid
            self.thumbnail_grid.addWidget(thumbnail, row, col)
            self.thumbnail_widgets[card.id] = thumbnail

            # Update grid position
            col += 1
            if col >= cols_per_row:
                col = 0
                row += 1

        # Start background thumbnail generation
        self.thumbnail_thread = ThumbnailGenerator(cards, self.renderer)
        self.thumbnail_thread.thumbnail_ready.connect(self.on_thumbnail_ready)
        self.thumbnail_thread.start()

    def on_thumbnail_ready(self, card_id, image_buffer):
        """Handle thumbnail generation completion"""
        # Only update if the widget still exists
        if card_id in self.thumbnail_widgets:
            self.thumbnail_widgets[card_id].set_image(image_buffer)

    def on_thumbnail_clicked(self, card):
        """Handle thumbnail click"""
        self.card_clicked.emit(card)

    def add_new_card(self):
        """Add a new card to this encounter set"""
        # Import here to avoid circular imports
        import shoggoth
        shoggoth.app.open_new_card_dialog()

    def delete_encounter_set(self):
        """Delete this encounter set"""
        reply = QMessageBox.question(
            self,
            tr("DLG_DELETE_ENCOUNTER_SET"),
            tr("CONFIRM_DELETE_ENCOUNTER_SET").format(name=self.encounter_set.name, count=len(self.encounter_set.cards)),
            QMessageBox.Yes | QMessageBox.No,
            QMessageBox.No
        )

        if reply == QMessageBox.Yes:
            # TODO: Implement deletion
            QMessageBox.information(self, tr("DLG_NOT_IMPLEMENTED"), tr("MSG_ENCOUNTER_SET_DELETION_NOT_IMPLEMENTED"))

    def cleanup(self):
        """Cleanup resources when editor is closed"""
        if self.thumbnail_thread and self.thumbnail_thread.isRunning():
            # Disconnect signal to prevent updates after cleanup
            try:
                self.thumbnail_thread.thumbnail_ready.disconnect(self.on_thumbnail_ready)
            except:
                pass  # Already disconnected

            # Stop the thread
            self.thumbnail_thread.stop()

            # Wait for it to finish (with timeout)
            self.thumbnail_thread.wait(1000)  # Wait max 1 second

            # If still running, terminate it
            if self.thumbnail_thread.isRunning():
                self.thumbnail_thread.terminate()
                self.thumbnail_thread.wait()

            self.thumbnail_thread = None

    def __del__(self):
        """Destructor - ensure cleanup happens"""
        self.cleanup()



"""
Main window implementation for Shoggoth using PySide6
"""
from PySide6.QtWidgets import (
    QMainWindow, QWidget, QVBoxLayout, QHBoxLayout, QSplitter,
    QTreeWidget, QTreeWidgetItem, QLabel, QMenuBar, QMenu,
    QFileDialog, QMessageBox, QStatusBar, QScrollArea, QDialog,
    QLineEdit, QPushButton, QDialogButtonBox, QTextBrowser
)
from PySide6.QtCore import QUrl, Qt, QTimer, Signal, Slot
from PySide6.QtGui import (
    QPixmap, QAction, QImage, QKeySequence, QShortcut, QIcon, QDesktopServices
)
from pathlib import Path
import json
import threading
from io import BytesIO

import shoggoth
from shoggoth.project import Project, Translation
from shoggoth.renderer import CardRenderer
from shoggoth.file_monitor import CardFileMonitor
from shoggoth.files import defaults_dir, asset_dir, font_dir, overlay_dir, root_dir
from shoggoth.ui.preview_widget import ImprovedCardPreview
from shoggoth.ui.goto_dialog import GotoCardDialog
from shoggoth.ui.encounter_editor import EncounterSetEditor
from shoggoth.i18n import get_available_languages, load_language, get_current_language, tr
from shoggoth.ui.card_editor import CardEditor


class DraggableTreeWidget(QTreeWidget):
    """Tree widget with custom drag and drop for card organization"""

    def __init__(self, file_browser, parent=None):
        super().__init__(parent)
        self.file_browser = file_browser
        self.setDragEnabled(True)
        self.setAcceptDrops(True)
        self.setDragDropMode(QTreeWidget.DragDrop)
        self.setDefaultDropAction(Qt.MoveAction)

    def _get_item_data(self, item):
        """Get the user data from a tree item"""
        if item is None:
            return None
        return item.data(0, Qt.UserRole)

    def _is_ancestor_of(self, potential_ancestor, item):
        """Check if potential_ancestor is an ancestor of item"""
        parent = item.parent()
        while parent is not None:
            if parent is potential_ancestor:
                return True
            parent = parent.parent()
        return False

    def _get_drop_target_type(self, target_item):
        """Determine what kind of drop target this is and return relevant info"""
        if target_item is None:
            return None, None

        data = self._get_item_data(target_item)
        if not data:
            return None, None

        item_type = data.get('type')
        item_data = data.get('data')

        # Non-droppable targets
        if item_type in ('project', 'campaign_cards', 'player_cards', 'guide'):
            return None, None

        # Check for Guides category
        if item_type == 'category' and target_item.text(0) == 'Guides':
            return None, None

        # Encounter set - can drop cards here
        if item_type == 'encounter':
            return 'encounter', item_data

        # Story/Location/Encounter categories under encounter set
        if item_type in ('category', 'locations'):
            if item_data and hasattr(item_data, 'id'):
                # This is a category under an encounter set
                return 'encounter', item_data

        # Class categories (player cards)
        if item_type == 'category':
            cls = data.get('class')
            if cls:
                if cls == 'investigators':
                    return None, None  # Can't drop directly on Investigators
                elif cls == 'other':
                    return 'other', None
                else:
                    return 'class', cls

            # Investigator group
            investigator = data.get('investigator')
            if investigator:
                return 'investigator', investigator

        # Card - check if it's under an encounter or player section
        if item_type == 'card':
            # Can't drop on a card, but could drop on its parent
            return None, None

        return None, None

    def mimeTypes(self):
        return ['application/x-shoggoth-card']

    def mimeData(self, items):
        """Create mime data for dragged items"""
        from PySide6.QtCore import QMimeData, QByteArray
        mime_data = QMimeData()

        # Only allow dragging cards
        card_ids = []
        for item in items:
            data = self._get_item_data(item)
            if data and data.get('type') == 'card':
                card = data.get('data')
                if card:
                    card_ids.append(card.id)

        if card_ids:
            mime_data.setData('application/x-shoggoth-card',
                            QByteArray(','.join(card_ids).encode()))
        return mime_data

    def dragEnterEvent(self, event):
        if event.mimeData().hasFormat('application/x-shoggoth-card'):
            event.acceptProposedAction()
        else:
            event.ignore()

    def dragMoveEvent(self, event):
        if not event.mimeData().hasFormat('application/x-shoggoth-card'):
            event.ignore()
            return

        target_item = self.itemAt(event.position().toPoint())
        drop_type, _ = self._get_drop_target_type(target_item)

        if drop_type is None:
            event.ignore()
            return

        # Check if dragging onto ancestor
        card_ids = event.mimeData().data('application/x-shoggoth-card').data().decode().split(',')
        for card_id in card_ids:
            node_id = f'card:{card_id}'
            if node_id in self.file_browser._node_map:
                dragged_item = self.file_browser._node_map[node_id]
                if self._is_ancestor_of(target_item, dragged_item):
                    event.ignore()
                    return

        event.acceptProposedAction()

    def dropEvent(self, event):
        if not event.mimeData().hasFormat('application/x-shoggoth-card'):
            event.ignore()
            return

        target_item = self.itemAt(event.position().toPoint())
        drop_type, drop_data = self._get_drop_target_type(target_item)

        if drop_type is None:
            event.ignore()
            return

        card_ids = event.mimeData().data('application/x-shoggoth-card').data().decode().split(',')

        for card_id in card_ids:
            # Find the project that contains this card
            card = None
            for project in self.file_browser._projects:
                card = project.get_card(card_id)
                if card:
                    break

            if not card:
                continue

            # Check ancestor constraint
            node_id = f'card:{card_id}'
            if node_id in self.file_browser._node_map:
                dragged_item = self.file_browser._node_map[node_id]
                if self._is_ancestor_of(target_item, dragged_item):
                    continue

            # Apply the drop action
            if drop_type == 'encounter':
                # Moving to an encounter set
                card.set('encounter_set', drop_data.id)

            elif drop_type == 'class':
                # Moving to a class category
                card.set('encounter_set', None)
                card.set('investigator', None)
                card.front.set('classes', [drop_data])
                if card.back.get('type') == 'encounter':
                    card.back.set('type', 'player')

            elif drop_type == 'other':
                # Moving to "Other" category
                card.set('encounter_set', None)
                card.set('investigator', None)
                card.front.set('classes', ['guardian', 'seeker'])
                if card.back.get('type') == 'encounter':
                    card.back.set('type', 'player')

            elif drop_type == 'investigator':
                # Moving to an investigator group
                card.set('encounter_set', None)
                card.set('investigator', drop_data)

        # Refresh the tree to reflect changes
        # Use accept() instead of acceptProposedAction() to prevent Qt from
        # performing its own item move - we handle tree updates via refresh_tree()
        event.accept()
        shoggoth.app.refresh_tree()


class FileBrowser(QWidget):
    """File browser widget showing project files"""

    card_selected = Signal(object)  # Emits card object
    encounter_selected = Signal(object)  # Emits encounter object
    project_selected = Signal(object)  # Emits project object
    guide_selected = Signal(object)  # Emits guide object
    locations_selected = Signal(object)  # Emits encounter set for location view
    active_project_changed = Signal(object)  # Emits project when active project changes

    def __init__(self):
        super().__init__()
        layout = QVBoxLayout()

        # Title
        title = QLabel(tr("TREE_CARDS"))
        title.setStyleSheet("font-size: 20pt; font-weight: bold;")
        layout.addWidget(title)

        # Tree widget with drag and drop support
        self.tree = DraggableTreeWidget(self)
        self.tree.setHeaderHidden(True)
        self.tree.itemClicked.connect(self.on_item_clicked)
        self.tree.setContextMenuPolicy(Qt.CustomContextMenu)
        self.tree.customContextMenuRequested.connect(self.on_context_menu)
        layout.addWidget(self.tree)

        self.setLayout(layout)
        self._projects = []  # List of open projects
        self._active_project = None  # The active project for operations
        self._node_map = {}  # Maps node_id -> QTreeWidgetItem for fast lookup

        # Context menu handler
        from shoggoth.ui.tree_context_menu import TreeContextMenu
        self.context_menu = TreeContextMenu(self)

    @property
    def _project(self):
        """Compatibility property - returns active project"""
        return self._active_project

    def set_project(self, project):
        """Set a single project (clears others) - for backward compatibility"""
        self._projects = [project] if project else []
        self._active_project = project
        self._node_map.clear()
        self._full_rebuild()

    def add_project(self, project):
        """Add a project to the browser. If first project, make it active."""
        if project in self._projects:
            # Already open - just make it active
            self.set_active_project(project)
            return

        self._projects.append(project)
        if self._active_project is None:
            self._active_project = project

        self._full_rebuild()
        self.active_project_changed.emit(self._active_project)

    def remove_project(self, project):
        """Remove a project from the browser"""
        if project not in self._projects:
            return

        self._projects.remove(project)

        # If we removed the active project, select another
        if self._active_project == project:
            self._active_project = self._projects[0] if self._projects else None
            self.active_project_changed.emit(self._active_project)

        self._full_rebuild()

    def set_active_project(self, project):
        """Set the active project for operations"""
        if project not in self._projects:
            return
        if self._active_project == project:
            return

        self._active_project = project
        self._update_active_project_display()
        self.active_project_changed.emit(project)

    def _update_active_project_display(self):
        """Update visual indicators for active project"""
        # Update all project nodes to show/hide bold
        for project in self._projects:
            node_id = f'project:{project.file_path}'
            if node_id in self._node_map:
                item = self._node_map[node_id]
                font = item.font(0)
                font.setBold(project == self._active_project)
                item.setFont(0, font)

    def get_project_for_card(self, card):
        """Find which project a card belongs to"""
        for project in self._projects:
            if project.get_card(card.id):
                return project
        return None

    def refresh(self):
        """Smart refresh - only update what has changed"""
        if not self._projects:
            self.tree.clear()
            self._node_map.clear()
            return

        # If tree is empty, do a full rebuild
        if self.tree.topLevelItemCount() == 0:
            self._full_rebuild()
            return

        # Build desired tree specification for all projects
        desired_specs = self._build_all_tree_specs()

        # Remove extra projects if any were closed
        while self.tree.topLevelItemCount() > len(desired_specs):
            removed_item = self.tree.takeTopLevelItem(self.tree.topLevelItemCount() - 1)
            self._remove_from_node_map(removed_item)

        # Apply incremental updates to each project root
        for i, spec in enumerate(desired_specs):
            if i < self.tree.topLevelItemCount():
                root_item = self.tree.topLevelItem(i)
                self._sync_tree_node(root_item, spec)
            else:
                # New project added
                root_item = self._create_tree_item(spec)
                self.tree.addTopLevelItem(root_item)
                root_item.setExpanded(True)

    def _full_rebuild(self):
        """Do a complete tree rebuild (used for initial load or project change)"""
        self.tree.clear()
        self._node_map.clear()

        if not self._projects:
            return

        for project in self._projects:
            spec = self._build_tree_spec(project)
            root_item = self._create_tree_item(spec)
            self.tree.addTopLevelItem(root_item)
            root_item.setExpanded(True)

            # Set bold for active project
            if project == self._active_project:
                font = root_item.font(0)
                font.setBold(True)
                root_item.setFont(0, font)

    def _build_all_tree_specs(self):
        """Build tree specs for all open projects"""
        return [self._build_tree_spec(project) for project in self._projects]

    def _build_tree_spec(self, project=None):
        """Build a specification of the desired tree state"""
        if project is None:
            project = self._active_project
        if not project:
            return None

        # Root node — translation projects get a distinct node_id and label
        node_id_path = getattr(project, '_node_id_path', project.file_path)
        translation = getattr(project, '_translation', None)
        label = (f"{translation.language} translation of {project['name']}"
                 if translation else project['name'])

        root_spec = {
            'node_id': f'project:{node_id_path}',
            'text': label,
            'type': 'project',
            'data': project,
            'icon': None,
            'children': []
        }

        # Determine if we need campaign/player split
        has_encounters = bool(project.encounter_sets)
        has_player_cards = any(c for c in project.cards if not c.encounter)

        if has_encounters and has_player_cards:
            campaign_spec = {
                'node_id': f'category:campaign_cards:{project.file_path}',
                'text': tr('TREE_CAMPAIGN_CARDS'),
                'type': 'campaign_cards',
                'data': project,
                'icon': None,
                'children': []
            }
            player_spec = {
                'node_id': f'category:player_cards:{project.file_path}',
                'text': tr('TREE_PLAYER_CARDS'),
                'type': 'player_cards',
                'data': project,
                'icon': None,
                'children': []
            }
            root_spec['children'].append(campaign_spec)
            root_spec['children'].append(player_spec)
        else:
            campaign_spec = player_spec = root_spec

        # Add encounter sets
        for encounter_set in project.encounter_sets:
            e_spec = {
                'node_id': f'encounter:{encounter_set.name}',
                'text': encounter_set.name,
                'type': 'encounter',
                'data': encounter_set,
                'icon': None,
                'children': []
            }

            story_spec = {
                'node_id': f'category:{encounter_set.name}:story',
                'text': tr('TREE_STORY'),
                'type': 'category',
                'data': encounter_set,
                'icon': None,
                'children': []
            }
            location_spec = {
                'node_id': f'locations:{encounter_set.name}',
                'text': tr('TREE_LOCATIONS'),
                'type': 'locations',
                'data': encounter_set,
                'icon': None,
                'children': []
            }
            encounter_cat_spec = {
                'node_id': f'category:{encounter_set.name}:encounter',
                'text': tr('TREE_ENCOUNTER'),
                'type': 'category',
                'data': encounter_set,
                'icon': None,
                'children': []
            }
            # Add cards to appropriate categories
            for card in encounter_set.cards:
                card_spec = self._build_card_spec(card)

                if card.front.get('type') == 'location':
                    location_spec['children'].append(card_spec)
                elif card.back.get('type') == 'encounter':
                    encounter_cat_spec['children'].append(card_spec)
                else:
                    story_spec['children'].append(card_spec)

            # If only encounter cards exist (no story or location), show cards directly
            if not story_spec['children'] and not location_spec['children']:
                e_spec['children'] = encounter_cat_spec['children']
            else:
                e_spec['children'] = [story_spec, location_spec, encounter_cat_spec]

            campaign_spec['children'].append(e_spec)

        # Add player cards
        class_labels = {
            'investigators': tr('TREE_INVESTIGATORS'),
            'seeker': tr('CLASS_SEEKER'),
            'rogue': tr('CLASS_ROGUE'),
            'guardian': tr('CLASS_GUARDIAN'),
            'mystic': tr('CLASS_MYSTIC'),
            'survivor': tr('CLASS_SURVIVOR'),
            'neutral': tr('CLASS_NEUTRAL'),
            'other': tr('CLASS_OTHER'),
        }
        classes_with_icons = {'guardian', 'seeker', 'rogue', 'mystic', 'survivor'}

        class_specs = {}
        for cls in ['investigators', 'seeker', 'rogue', 'guardian', 'mystic', 'survivor', 'neutral', 'other']:
            icon_path = None
            if cls in classes_with_icons:
                path = overlay_dir / f"class_symbol_{cls}.png"
                if path.exists():
                    icon_path = str(path)

            class_spec = {
                'node_id': f'class:{cls}:{project.file_path}',
                'text': class_labels[cls],
                'type': 'category',
                'data': project,
                'class': cls,
                'icon': icon_path,
                'children': []
            }
            class_specs[cls] = class_spec
            player_spec['children'].append(class_spec)

        investigator_specs = {}

        for card in project.player_cards:
            if group := card.data.get('investigator', False):
                if group not in investigator_specs:
                    inv_spec = {
                        'node_id': f'investigator:{group}:{project.file_path}',
                        'text': group,
                        'type': 'category',
                        'data': project,
                        'investigator': group,
                        'icon': None,
                        'children': []
                    }
                    investigator_specs[group] = inv_spec
                    class_specs['investigators']['children'].append(inv_spec)
                target_spec = investigator_specs[group]
            else:
                card_class = card.get_class() or 'other'
                target_spec = class_specs.get(card_class, class_specs['other'])

            card_spec = self._build_card_spec(card, include_level=True)
            target_spec['children'].append(card_spec)

        # Add guides
        if project.guides:
            guide_parent = {
                'node_id': f'category:guides:{project.file_path}',
                'text': tr('TREE_GUIDES'),
                'type': 'category',
                'data': project,
                'icon': None,
                'children': []
            }
            for guide in project.guides:
                guide_spec = {
                    'node_id': f'guide:{guide.id}',
                    'text': guide.name,
                    'type': 'guide',
                    'data': guide,
                    'icon': None,
                    'children': []
                }
                guide_parent['children'].append(guide_spec)
            root_spec['children'].append(guide_parent)

        return root_spec

    def _build_card_spec(self, card, include_level=False):
        """Build a specification for a card node"""
        if include_level and str(card.front.get('level', '0')) != '0':
            display_name = f'{card.name} ({card.front.get("level")})'
        else:
            display_name = card.name

        # Add dirty indicator
        if card.dirty:
            display_name = '● ' + display_name

        return {
            'node_id': f'card:{card.id}',
            'text': display_name,
            'type': 'card',
            'data': card,
            'icon': None,
            'children': []
        }

    def _create_tree_item(self, spec):
        """Create a QTreeWidgetItem from a spec, recursively"""
        item = QTreeWidgetItem([spec['text']])

        # Build user data
        user_data = {'type': spec['type'], 'data': spec['data'], 'node_id': spec['node_id']}
        if 'class' in spec:
            user_data['class'] = spec['class']
        if 'investigator' in spec:
            user_data['investigator'] = spec['investigator']
        item.setData(0, Qt.UserRole, user_data)

        # Set icon if specified
        if spec.get('icon'):
            item.setIcon(0, QIcon(spec['icon']))

        # Store in node map for fast lookup
        self._node_map[spec['node_id']] = item

        # Create children
        for child_spec in spec.get('children', []):
            child_item = self._create_tree_item(child_spec)
            item.addChild(child_item)

        return item

    def _sync_tree_node(self, item, spec):
        """Synchronize an existing tree item with a spec, updating only what changed"""
        if spec is None:
            return

        # Update text if changed
        if item.text(0) != spec['text']:
            item.setText(0, spec['text'])

        # Update user data
        user_data = {'type': spec['type'], 'data': spec['data'], 'node_id': spec['node_id']}
        if 'class' in spec:
            user_data['class'] = spec['class']
        if 'investigator' in spec:
            user_data['investigator'] = spec['investigator']
        item.setData(0, Qt.UserRole, user_data)

        # Update node map
        self._node_map[spec['node_id']] = item

        # Build maps of current and desired children
        current_children = {}
        for i in range(item.childCount()):
            child = item.child(i)
            child_data = child.data(0, Qt.UserRole)
            if child_data:
                # Build node_id from current item
                child_id = self._get_node_id_from_item(child)
                if child_id:
                    current_children[child_id] = (i, child)

        desired_children = {child_spec['node_id']: child_spec for child_spec in spec.get('children', [])}
        desired_order = [child_spec['node_id'] for child_spec in spec.get('children', [])]

        # Find nodes to remove (in current but not in desired)
        to_remove = set(current_children.keys()) - set(desired_children.keys())

        # Find nodes to add (in desired but not in current)
        to_add = set(desired_children.keys()) - set(current_children.keys())

        # Find nodes to update (in both)
        to_update = set(current_children.keys()) & set(desired_children.keys())

        # Remove nodes (in reverse order to maintain indices)
        remove_indices = sorted([current_children[node_id][0] for node_id in to_remove], reverse=True)
        for idx in remove_indices:
            removed_item = item.takeChild(idx)
            # Clean up node map
            self._remove_from_node_map(removed_item)

        # Update existing nodes
        for node_id in to_update:
            _, child_item = current_children[node_id]
            child_spec = desired_children[node_id]
            self._sync_tree_node(child_item, child_spec)

        # Add new nodes
        for node_id in to_add:
            child_spec = desired_children[node_id]
            new_item = self._create_tree_item(child_spec)
            item.addChild(new_item)  # Add at end for now

        # Reorder children to match desired order
        self._reorder_children(item, desired_order)

    def _get_node_id_from_item(self, item):
        """Extract or construct a node_id from an existing tree item"""
        data = item.data(0, Qt.UserRole)
        if not data:
            return None

        # Use the stored node_id if available (avoids depending on translated display text)
        if 'node_id' in data:
            return data['node_id']

        item_type = data.get('type')
        item_data = data.get('data')

        # Get project path for unique IDs
        project_path = ''
        if item_data and hasattr(item_data, 'file_path'):
            project_path = item_data.file_path
        elif item_data and hasattr(item_data, 'expansion') and item_data.expansion:
            project_path = item_data.expansion.file_path

        if item_type == 'card' and item_data:
            return f'card:{item_data.id}'
        elif item_type == 'guide' and item_data:
            return f'guide:{item_data.id}'
        elif item_type == 'encounter' and item_data:
            return f'encounter:{item_data.name}'
        elif item_type == 'project' and item_data:
            return f'project:{item_data.file_path}'
        elif item_type == 'locations' and item_data:
            return f'locations:{item_data.name}'
        elif item_type == 'campaign_cards' and item_data:
            return f'category:campaign_cards:{item_data.file_path}'
        elif item_type == 'player_cards' and item_data:
            return f'category:player_cards:{item_data.file_path}'
        elif item_type == 'category':
            if data.get('class') and item_data:
                return f'class:{data["class"]}:{item_data.file_path}'
            elif data.get('investigator') and item_data:
                return f'investigator:{data["investigator"]}:{item_data.file_path}'
            elif item_data and hasattr(item_data, 'file_path'):
                # Project-level category (Guides) - item_data is Project
                if item.text(0) == 'Guides':
                    return f'category:guides:{item_data.file_path}'
            elif item_data and hasattr(item_data, 'name') and not hasattr(item_data, 'file_path'):
                # Encounter subcategory (Story/Encounter) - item_data is EncounterSet (no file_path)
                text = item.text(0).lower()
                return f'category:{item_data.name}:{text}'
        return None

    def _remove_from_node_map(self, item):
        """Recursively remove an item and its children from the node map"""
        node_id = self._get_node_id_from_item(item)
        if node_id and node_id in self._node_map:
            del self._node_map[node_id]

        for i in range(item.childCount()):
            self._remove_from_node_map(item.child(i))

    def _reorder_children(self, parent_item, desired_order):
        """Reorder children of a parent item to match desired order"""
        if not desired_order:
            return

        # Build current order map
        current_order = []
        for i in range(parent_item.childCount()):
            child = parent_item.child(i)
            node_id = self._get_node_id_from_item(child)
            current_order.append(node_id)

        # Check if reordering is needed
        if current_order == desired_order:
            return

        # Remove all children while preserving expansion state
        children_with_expansion = []
        while parent_item.childCount() > 0:
            child = parent_item.takeChild(0)
            node_id = self._get_node_id_from_item(child)
            children_with_expansion.append((node_id, child, child.isExpanded()))

        # Create lookup
        child_lookup = {node_id: (child, expanded) for node_id, child, expanded in children_with_expansion}

        # Re-add in correct order
        for node_id in desired_order:
            if node_id in child_lookup:
                child, expanded = child_lookup[node_id]
                parent_item.addChild(child)
                child.setExpanded(expanded)

    def update_card_node(self, card_id):
        """Update a single card node by its ID without rebuilding the tree"""
        node_id = f'card:{card_id}'
        if node_id not in self._node_map:
            # Card not in tree, might need full refresh
            return False

        item = self._node_map[node_id]
        data = item.data(0, Qt.UserRole)
        if not data or data.get('type') != 'card':
            return False

        card = data.get('data')
        print('update_card_node, card', card)
        if not card:
            return False

        # Determine if this is a player card (needs level in name)
        include_level = not card.encounter

        # Build new display name
        if include_level and str(card.front.get('level', '0')) != '0':
            display_name = f'{card.name} ({card.front.get("level")})'
        else:
            display_name = card.name

        # Add dirty indicator
        if card.dirty:
            print('update_card_node, dirty:', card.dirty)
            display_name = '● ' + display_name
        else:
            print('update_card_node, false dirty:', card.dirty)

        # Update only if changed
        if item.text(0) != display_name:
            item.setText(0, display_name)

        return True

    def get_card_item(self, card_id):
        """Get the tree item for a card by ID"""
        node_id = f'card:{card_id}'
        return self._node_map.get(node_id)

    def on_item_clicked(self, item, column):
        """Handle tree item click"""
        data = item.data(0, Qt.UserRole)
        if not data:
            return

        item_type = data['type']
        item_data = data['data']

        if item_type == 'project':
            # Clicking a project sets it as active
            self.set_active_project(item_data)
            self.project_selected.emit(item_data)
        elif item_type == 'encounter':
            self.encounter_selected.emit(item_data)
        elif item_type == 'card':
            self.card_selected.emit(item_data)
        elif item_type == 'guide':
            self.guide_selected.emit(item_data)
        elif item_type == 'locations':
            self.locations_selected.emit(item_data)

    def on_context_menu(self, position):
        """Handle context menu request"""
        item = self.tree.itemAt(position)
        if item:
            # Convert position to global coordinates
            global_pos = self.tree.viewport().mapToGlobal(position)
            self.context_menu.show_context_menu(item, global_pos)



class ShoggothMainWindow(QMainWindow):
    """Main window for Shoggoth application"""

    # Signal for file changes (emitted from background thread, handled on main thread)
    file_changed_signal = Signal(str)

    # Signal for render results (version, front_image, back_image)
    render_result_signal = Signal(int, object, object)

    def __init__(self):
        super().__init__()

        # Connect file change signal to handler (for thread-safe UI updates)
        self.file_changed_signal.connect(self._handle_file_changed)

        # Connect render result signal to handler
        self.render_result_signal.connect(self._handle_render_result)

        # Settings manager - must be before shoggoth.app assignment
        from shoggoth.settings import SettingsManager
        self.config = SettingsManager()

        shoggoth.app = self

        self.setWindowTitle(tr("APP_TITLE"))
        self.setMinimumSize(1400, 900)

        # Set application icon
        icon_path = asset_dir / "elder_sign_neon.png"
        if icon_path.exists():
            self.setWindowIcon(QIcon(str(icon_path)))

        # Initialize components
        self.open_projects = []  # List of all open projects
        self.active_project = None  # The currently active project
        self.current_card = None
        self.current_editor = None
        self.card_renderer = CardRenderer()
        self.card_file_monitor = None

        # Preview rendering with debounce
        self.render_timer = QTimer()
        self.render_timer.setSingleShot(True)
        self.render_timer.timeout.connect(self._start_background_render)
        self.render_version = 0  # Tracks render requests, stale results are discarded

        # Navigation history (browser-like back/forward)
        self._nav_history = []  # List of (type, id) tuples
        self._nav_index = -1    # Current position in history
        self._nav_navigating = False  # Prevent recursive history updates during navigation

        # Load settings
        self.load_settings()

        # Initialize update manager (before setup_ui so menu can reference it)
        from shoggoth.ui.updater_ui import UpdateManager
        self.update_manager = UpdateManager(self.config, self)

        # Setup UI
        self.setup_ui()

        # Setup file monitoring
        self.setup_file_monitoring()

        # Restore session
        self.restore_session()

        # Check for updates after UI is ready (deferred)
        QTimer.singleShot(2000, self._check_for_updates_startup)

    def setup_ui(self):
        """Setup the user interface"""
        # Create menu bar
        self.create_menus()

        # Create central widget
        central = QWidget()
        self.setCentralWidget(central)

        # Main layout - just file browser + content area
        main_layout = QHBoxLayout()

        # Create main splitter (2 panels: browser + content)
        self.main_splitter = QSplitter(Qt.Horizontal)

        # Left panel - File browser
        self.file_browser = FileBrowser()
        self.file_browser.card_selected.connect(self.show_card)
        self.file_browser.encounter_selected.connect(self.show_encounter)
        self.file_browser.project_selected.connect(self.show_project)
        self.file_browser.guide_selected.connect(self.show_guide)
        self.file_browser.locations_selected.connect(self.show_locations)
        self.file_browser.active_project_changed.connect(self._on_active_project_changed)
        self.main_splitter.addWidget(self.file_browser)

        # Right panel - Content area (will contain editor + preview for cards)
        self.content_container = QWidget()
        self.content_layout = QVBoxLayout()
        self.content_layout.setContentsMargins(0, 0, 0, 0)
        self.content_container.setLayout(self.content_layout)
        self.main_splitter.addWidget(self.content_container)

        # Create the editor and preview containers (hidden by default)
        # Editor container
        self.editor_container = QScrollArea()
        self.editor_container.setWidgetResizable(True)
        self.editor_widget = QLabel(tr("MSG_OPEN_OR_CREATE"))
        self.editor_widget.setAlignment(Qt.AlignCenter)
        self.editor_container.setWidget(self.editor_widget)

        # Preview container (detachable)
        from PySide6.QtWidgets import QDockWidget
        self.preview_dock = QDockWidget(tr("CARD_PREVIEW"), self)
        self.preview_dock.setFeatures(
            QDockWidget.DockWidgetFloatable |
            QDockWidget.DockWidgetMovable |
            QDockWidget.DockWidgetClosable
        )
        self.card_preview = ImprovedCardPreview()
        self.preview_dock.setWidget(self.card_preview)

        # Add dock widget to right side (hidden initially)
        self.addDockWidget(Qt.RightDockWidgetArea, self.preview_dock)
        self.preview_dock.hide()

        # Set initial splitter sizes
        self.main_splitter.setSizes([300, 700])

        main_layout.addWidget(self.main_splitter)
        central.setLayout(main_layout)

        # Status bar
        self.status_bar = QStatusBar()
        self.setStatusBar(self.status_bar)
        self.status_bar.showMessage(tr("STATUS_READY"))

        # Setup global shortcuts
        self.setup_shortcuts()

    def setup_shortcuts(self):
        """Setup global keyboard shortcuts"""
        # Ctrl+P - Go to Card
        # TODO: Re-enable shortcuts once issues are resolved
        # goto_shortcut = QShortcut(QKeySequence("Ctrl+P"), self)
        # goto_shortcut.activated.connect(self.show_goto_dialog)
        pass

    def show_goto_dialog(self):
        """Show the Go to Card dialog"""
        if not self.active_project:
            QMessageBox.information(self, tr("DLG_NO_PROJECT"), tr("MSG_OPEN_PROJECT_FIRST"))
            return

        dialog = GotoCardDialog(self.active_project, self)
        dialog.card_selected.connect(self.on_goto_card_selected)
        dialog.exec()

    def on_goto_card_selected(self, card):
        """Handle card selection from goto dialog"""
        # Check if it's a guide
        if hasattr(card, 'is_guide') and card.is_guide:
            self.show_guide(card.guide)
            # Find and select in tree
            self.select_item_in_tree(card.id)
        else:
            # It's a card
            self.show_card(card)
            # Find and select in tree
            self.select_item_in_tree(card.id)

    def select_item_in_tree(self, item_id):
        """Find and select an item in the tree by ID, expanding parents"""
        tree = self.file_browser.tree
        for item in self.iterate_tree_items(tree.invisibleRootItem()):
            data = item.data(0, Qt.UserRole)
            if data and data.get('data'):
                if hasattr(data['data'], 'id') and str(data['data'].id) == str(item_id):
                    # Expand all parent items
                    self.expand_to_item(item)
                    # Select and scroll to the item
                    tree.setCurrentItem(item)
                    tree.scrollToItem(item)
                    return True
        return False

    def iterate_tree_items(self, root_item):
        """Recursively iterate all items in tree"""
        for i in range(root_item.childCount()):
            child = root_item.child(i)
            yield child
            yield from self.iterate_tree_items(child)

    def expand_to_item(self, item):
        """Expand all parent items to make item visible"""
        parent = item.parent()
        while parent:
            parent.setExpanded(True)
            parent = parent.parent()

    def create_menus(self):
        """Create comprehensive menu bar based on shoggoth.kv"""
        menubar = self.menuBar()

        # ==================== FILE MENU ====================
        file_menu = menubar.addMenu(tr("MENU_FILE"))

        # Open Project
        open_action = QAction(tr("MENU_OPEN_PROJECT"), self)
        open_action.setShortcut("Ctrl+O")
        open_action.triggered.connect(self.open_project_dialog)
        file_menu.addAction(open_action)

        # New Project
        new_action = QAction(tr("MENU_NEW_PROJECT"), self)
        new_action.triggered.connect(self.new_project_dialog)
        file_menu.addAction(new_action)

        # Close Project
        close_project_action = QAction(tr("MENU_CLOSE_PROJECT"), self)
        close_project_action.triggered.connect(lambda: self.close_project())
        file_menu.addAction(close_project_action)

        file_menu.addSeparator()

        # Save
        save_action = QAction(tr("MENU_SAVE"), self)
        save_action.setShortcut("Ctrl+S")
        save_action.triggered.connect(self.save_changes)
        file_menu.addAction(save_action)

        # New Card
        new_card_action = QAction(tr("MENU_NEW_CARD"), self)
        new_card_action.setShortcut("Ctrl+N")
        new_card_action.triggered.connect(self.new_card_dialog)
        file_menu.addAction(new_card_action)

        # Go to Card
        goto_card_action = QAction(tr("MENU_GOTO_CARD"), self)
        goto_card_action.setShortcut("Ctrl+P")
        goto_card_action.triggered.connect(self.show_goto_dialog)
        file_menu.addAction(goto_card_action)

        file_menu.addSeparator()

        # Gather images
        gather_action = QAction(tr("MENU_GATHER_IMAGES"), self)
        gather_action.triggered.connect(self.gather_images)
        file_menu.addAction(gather_action)

        gather_update_action = QAction(tr("MENU_GATHER_UPDATE"), self)
        gather_update_action.triggered.connect(lambda: self.gather_images(update=True))
        file_menu.addAction(gather_update_action)

        file_menu.addSeparator()

        # Export Current Card
        export_current = QAction(tr("MENU_EXPORT_CURRENT"), self)
        export_current.setShortcut("Ctrl+E")
        export_current.triggered.connect(lambda: self.export_current())
        file_menu.addAction(export_current)

        # Export All Cards
        export_all = QAction(tr("MENU_EXPORT_ALL"), self)
        export_all.setShortcut("Ctrl+Shift+E")
        export_all.triggered.connect(self.export_all)
        file_menu.addAction(export_all)

        file_menu.addSeparator()

        # Settings
        settings_action = QAction(tr("MENU_SETTINGS"), self)
        settings_action.triggered.connect(self.open_settings)
        file_menu.addAction(settings_action)

        file_menu.addSeparator()

        # Exit
        exit_action = QAction(tr("MENU_EXIT"), self)
        exit_action.triggered.connect(self.close)
        file_menu.addAction(exit_action)

        # ==================== PROJECT MENU ====================
        project_menu = menubar.addMenu(tr("MENU_PROJECT"))

        # Add Guide
        auto_enumerate_action = QAction(tr("MENU_AUTO_ENUMERATE"), self)
        auto_enumerate_action.setShortcut("Ctrl+M")
        auto_enumerate_action.triggered.connect(self.auto_enumerate)
        project_menu.addAction(auto_enumerate_action)

        # Add Guide
        add_guide_action = QAction(tr("MENU_ADD_GUIDE"), self)
        add_guide_action.triggered.connect(self.add_guide)
        project_menu.addAction(add_guide_action)

        # Add Scenario template
        add_scenario_action = QAction(tr("MENU_ADD_SCENARIO"), self)
        add_scenario_action.triggered.connect(self.add_scenario_template)
        project_menu.addAction(add_scenario_action)

        # Add Campaign template
        add_campaign_action = QAction(tr("MENU_ADD_CAMPAIGN"), self)
        add_campaign_action.triggered.connect(self.add_campaign_template)
        project_menu.addAction(add_campaign_action)

        # Add Investigator template
        add_investigator_action = QAction(tr("MENU_ADD_INVESTIGATOR"), self)
        add_investigator_action.triggered.connect(self.add_investigator_template)
        project_menu.addAction(add_investigator_action)

        # Add Investigator Expansion template
        add_inv_exp_action = QAction(tr("MENU_ADD_INV_EXPANSION"), self)
        add_inv_exp_action.triggered.connect(self.add_investigator_expansion_template)
        project_menu.addAction(add_inv_exp_action)

        # ── Translation ──────────────────────────────────────────────────────
        project_menu.addSeparator()

        add_translation_action = QAction("Add Translation...", self)
        add_translation_action.triggered.connect(self.add_translation_dialog)
        project_menu.addAction(add_translation_action)

        load_translation_action = QAction("Load Translation...", self)
        load_translation_action.triggered.connect(self.load_translation_dialog)
        project_menu.addAction(load_translation_action)

        # ==================== EXPORT MENU ====================
        export_menu = menubar.addMenu(tr("MENU_EXPORT"))

        # Card To PDF
        card_pdf_action = QAction(tr("MENU_CARD_TO_PDF"), self)
        card_pdf_action.triggered.connect(self.export_card_to_pdf)
        export_menu.addAction(card_pdf_action)

        # Cards To PDF
        cards_pdf_action = QAction(tr("MENU_CARDS_TO_PDF"), self)
        cards_pdf_action.triggered.connect(self.export_cards_to_pdf)
        export_menu.addAction(cards_pdf_action)

        # Project To PDF
        project_pdf_action = QAction(tr("MENU_PROJECT_TO_PDF"), self)
        project_pdf_action.triggered.connect(self.export_project_to_pdf)
        export_menu.addAction(project_pdf_action)

        export_menu.addSeparator()

        # Export card to TTS object
        tts_card_action = QAction(tr("MENU_EXPORT_TTS_CARD"), self)
        tts_card_action.triggered.connect(self.export_card_to_tts)
        export_menu.addAction(tts_card_action)

        # Export all campaign cards to TTS object
        tts_campaign_action = QAction(tr("MENU_EXPORT_TTS_CAMPAIGN"), self)
        tts_campaign_action.triggered.connect(self.export_campaign_to_tts)
        export_menu.addAction(tts_campaign_action)

        # Export all player cards to TTS object
        tts_player_action = QAction(tr("MENU_EXPORT_TTS_PLAYER"), self)
        tts_player_action.triggered.connect(self.export_player_to_tts)
        export_menu.addAction(tts_player_action)

        # ==================== TOOLS MENU ====================
        tools_menu = menubar.addMenu(tr("MENU_TOOLS"))

        # Convert SE Project
        convert_se_action = QAction(tr("MENU_CONVERT_SE"), self)
        convert_se_action.triggered.connect(self.convert_strange_eons)
        tools_menu.addAction(convert_se_action)

        tools_menu.addSeparator()

        # Check for Updates
        check_updates_action = QAction(tr("MENU_CHECK_UPDATES"), self)
        check_updates_action.triggered.connect(self.update_manager.check_for_updates_manual)
        tools_menu.addAction(check_updates_action)

        # ==================== HELP MENU ====================
        help_menu = menubar.addMenu(tr("MENU_HELP"))

        # Text options
        text_options_action = QAction(tr("MENU_TEXT_OPTIONS"), self)
        text_options_action.triggered.connect(self.show_text_options)
        help_menu.addAction(text_options_action)

        # About
        about_action = QAction(tr("MENU_ABOUT"), self)
        about_action.triggered.connect(self.show_about)
        help_menu.addAction(about_action)

        asset_location_action = QAction(tr("MENU_ASSET_LOCATION"), self)
        asset_location_action.triggered.connect(self.open_asset_location)
        help_menu.addAction(asset_location_action)

        # View menu
        view_menu = menubar.addMenu(tr("MENU_VIEW"))

        # Toggle Preview
        self.toggle_preview_action = QAction(tr("MENU_SHOW_PREVIEW"), self)
        self.toggle_preview_action.setCheckable(True)
        self.toggle_preview_action.setChecked(False)
        self.toggle_preview_action.triggered.connect(self.toggle_preview)
        view_menu.addAction(self.toggle_preview_action)

        # ==================== LANGUAGE MENU ====================
        language_menu = menubar.addMenu(tr("MENU_LANGUAGE"))
        self.language_actions = []
        
        available_languages = get_available_languages()
        current_lang = self.config.get('Shoggoth', 'language', 'en')
        
        for lang_code, lang_name in available_languages.items():
            action = QAction(lang_name, self)
            action.setCheckable(True)
            action.setChecked(lang_code == current_lang)
            action.setData(lang_code)
            action.triggered.connect(lambda checked, code=lang_code: self.change_language(code))
            language_menu.addAction(action)
            self.language_actions.append(action)

    def toggle_preview(self, checked):
        """Toggle the preview dock visibility"""
        if checked:
            self.preview_dock.show()
        else:
            self.preview_dock.hide()

    def change_language(self, lang_code: str):
        """Change the application language"""
        # Update checkmarks in menu
        for action in self.language_actions:
            action.setChecked(action.data() == lang_code)
        
        # Save setting
        self.config.set('Shoggoth', 'language', lang_code)
        self.config.save()
        
        # Load the new language
        load_language(lang_code)
        
        # Show restart message
        QMessageBox.information(
            self,
            tr("DLG_LANGUAGE_CHANGED"),
            tr("MSG_RESTART_FOR_LANGUAGE")
        )

    def load_settings(self):
        """Load application settings"""
        settings_file = root_dir / 'shoggoth.json'
        if settings_file.exists():
            with open(settings_file, 'r') as f:
                self.settings = json.load(f)
        else:
            self.settings = {'session': {}, 'last_paths': {}}

    def save_settings(self):
        """Save application settings"""
        with open(root_dir / 'shoggoth.json', 'w') as f:
            json.dump(self.settings, f)

    def setup_file_monitoring(self):
        """Setup file system monitoring for assets and card files"""
        self.card_file_monitor = CardFileMonitor(asset_dir, self.on_file_changed)
        self.card_file_monitor.start()

    def restore_session(self):
        """Restore previous session"""
        session = self.settings.get('session', {})

        # Support for multiple open projects
        open_project_paths = session.get('open_projects', [])
        open_translation_paths = session.get('open_translations', [])
        active_project_path = session.get('active_project', session.get('project'))

        # Fallback for old sessions that stored only a single project path.
        # Only apply when there are no translations recorded; if the active path
        # matches a translation we must not open it as a plain project.
        if not open_project_paths and not open_translation_paths and active_project_path:
            open_project_paths = [active_project_path]

        for project_path in open_project_paths:
            try:
                self.open_project(project_path)
            except Exception as e:
                print(f"Error restoring project {project_path}: {e}")

        for trans_path in open_translation_paths:
            try:
                self.open_translation(trans_path)
            except Exception as e:
                print(f"Error restoring translation {trans_path}: {e}")

        # Set the active project — match by _node_id_path for translations,
        # file_path for regular projects.
        if active_project_path:
            for project in self.open_projects:
                path = getattr(project, '_node_id_path', project.file_path)
                if path == active_project_path:
                    self.file_browser.set_active_project(project)
                    break

        # Restore last selected element
        if last_id := session.get('last_id'):
            last_type = session.get('last_type', 'card')
            QTimer.singleShot(100, lambda: self._restore_last_element(last_id, last_type))

    def _save_session(self):
        """Save current session state"""
        regular = [p.file_path for p in self.open_projects
                   if not getattr(p, '_translation', None)]
        trans = [p._node_id_path for p in self.open_projects
                 if getattr(p, '_translation', None)]
        active_path = (getattr(self.active_project, '_node_id_path', self.active_project.file_path)
                       if self.active_project else None)
        self.settings['session']['open_projects'] = regular
        self.settings['session']['open_translations'] = trans
        self.settings['session']['active_project'] = active_path
        # Keep legacy 'project' key for backward compatibility
        self.settings['session']['project'] = active_path
        self.save_settings()

    def _restore_last_element(self, element_id, element_type):
        """Restore the last selected element by ID and type"""
        if not self.active_project:
            return

        element = None
        if element_type == 'card':
            element = self.active_project.get_card(element_id)
            if element:
                self.show_card(element)
        elif element_type == 'encounter':
            element = self.active_project.get_encounter_set(element_id)
            if element:
                self.show_encounter(element)
        elif element_type == 'guide':
            element = self.active_project.get_guide(element_id)
            if element:
                self.show_guide(element)
        elif element_type == 'locations':
            element = self.active_project.get_encounter_set(element_id)
            if element:
                self.show_locations(element)

        # Select in tree if element was found
        if element:
            self.select_item_in_tree(element_id)

    def _check_for_updates_startup(self):
        """Check for updates after startup (deferred to avoid blocking)"""
        if self.update_manager.should_check_for_updates():
            self.update_manager.check_for_updates_async()

    def open_project_dialog(self):
        """Show dialog to open a project"""
        file_path, _ = QFileDialog.getOpenFileName(
            self,
            tr("DLG_OPEN_PROJECT"),
            str(Path.home()),
            tr("FILTER_SHOGGOTH_PROJECTS")
        )
        if file_path:
            self.open_project(file_path)

    def open_project(self, file_path):
        """Open a project file"""
        try:
            # Check if project is already open
            for project in self.open_projects:
                if project.file_path == file_path:
                    # Already open - just make it active
                    self.file_browser.set_active_project(project)
                    self.status_bar.showMessage(tr("STATUS_SWITCHED_TO").format(name=project['name']))
                    return

            # Load new project
            project = Project.load(file_path)
            self.open_projects.append(project)
            self.active_project = project
            self.file_browser.add_project(project)
            self._save_session()
            # Clear navigation history for new project
            self._nav_history.clear()
            self._nav_index = -1
            self.status_bar.showMessage(tr("STATUS_OPENED").format(name=project['name']))
        except Exception as e:
            QMessageBox.critical(self, tr("DLG_ERROR"), tr("ERR_OPEN_PROJECT").format(error=e))

    def close_project(self, project=None):
        """Close a project"""
        if project is None:
            project = self.active_project
        if project is None:
            return

        # Check for unsaved changes in this project
        if self._project_has_unsaved_changes(project):
            msg_box = QMessageBox(self)
            msg_box.setWindowTitle(tr("DLG_UNSAVED_CHANGES"))
            msg_box.setText(tr("CONFIRM_SAVE_BEFORE_CLOSE").format(name=project['name']))
            msg_box.setIcon(QMessageBox.Question)
            save_btn = msg_box.addButton(tr("DLG_SAVE"), QMessageBox.AcceptRole)
            discard_btn = msg_box.addButton(tr("DLG_DISCARD"), QMessageBox.DestructiveRole)
            cancel_btn = msg_box.addButton(tr("DLG_CANCEL"), QMessageBox.RejectRole)
            msg_box.setDefaultButton(save_btn)
            msg_box.exec()
            
            if msg_box.clickedButton() == save_btn:
                project.save()
            elif msg_box.clickedButton() == cancel_btn:
                return

        # Remove from open projects
        if project in self.open_projects:
            self.open_projects.remove(project)

        # Update file browser
        self.file_browser.remove_project(project)

        # Update active project
        if self.active_project == project:
            self.active_project = self.open_projects[0] if self.open_projects else None

        self._save_session()
        self.status_bar.showMessage(tr("STATUS_CLOSED").format(name=project['name']))

    def _project_has_unsaved_changes(self, project):
        """Check if a specific project has unsaved changes"""
        for card in project.get_all_cards():
            if hasattr(card, 'dirty') and card.dirty:
                return True
        return False

    def _on_active_project_changed(self, project):
        """Handle active project change from file browser"""
        self.active_project = project
        if project:
            self.status_bar.showMessage(tr("STATUS_ACTIVE").format(name=project['name']))

    @property
    def current_project(self):
        """Backward compatibility property"""
        return self.active_project

    def new_project_dialog(self):
        """Show dialog to create a new project"""
        from shoggoth.ui.dialogs import NewProjectDialog
        dialog = NewProjectDialog(self)
        dialog.exec()

    def clear_editor(self):
        """Clear and cleanup the current editor"""
        # Clear all widgets in content layout
        while self.content_layout.count():
            item = self.content_layout.takeAt(0)
            widget = item.widget()
            if widget:
                # Call cleanup if the widget has it (recursively for nested widgets)
                self._cleanup_widget(widget)
                widget.setParent(None)
                widget.deleteLater()

    def _cleanup_widget(self, widget):
        """Recursively cleanup a widget and its children"""
        # Check if this widget has cleanup method
        if hasattr(widget, 'cleanup'):
            try:
                widget.cleanup()
            except Exception as e:
                print(f"Error during widget cleanup: {e}")

        # Recursively cleanup children
        for child in widget.findChildren(QWidget):
            if hasattr(child, 'cleanup'):
                try:
                    child.cleanup()
                except Exception as e:
                    print(f"Error during child cleanup: {e}")

    def save_changes(self):
        """Save the entire project"""
        if not self.active_project:
            return

        try:
            self.active_project.save()

            # Mark all cards as clean
            for card in self.active_project.get_all_cards():
                card.dirty = False
                if hasattr(card, 'front') and hasattr(card.front, 'dirty'):
                    card.front.dirty = False
                if hasattr(card, 'back') and hasattr(card.back, 'dirty'):
                    card.back.dirty = False

            # Update tree to remove dirty indicators
            self.file_browser.refresh()

            self.status_bar.showMessage(tr("STATUS_SAVED"), 3000)
        except Exception as e:
            QMessageBox.critical(self, tr("DLG_SAVE_ERROR"), tr("ERR_SAVE_PROJECT").format(error=e))

    def save_current(self):
        """Save only the current card"""
        if not self.current_card:
            return

        try:
            self.current_card.save()
            # Note: Card.save() already clears dirty flags and updates tree node
            self.status_bar.showMessage(tr("STATUS_SAVED_NAME").format(name=self.current_card.name), 3000)
        except Exception as e:
            QMessageBox.critical(self, tr("DLG_SAVE_ERROR"), tr("ERR_SAVE_CARD").format(error=e))

    def _get_export_size(self):
        """Return the size dict selected in export settings."""
        from shoggoth.settings import EXPORT_SIZES
        index = self.config.getint('Shoggoth', 'export_size', 1)
        index = min(index, len(EXPORT_SIZES) - 1)
        return EXPORT_SIZES[index][1]

    def export_all(self, bleed=None, format=None, quality=None, separate_versions=None):
        """Export all cards in the project using settings from preferences"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_LOADED"))
            return

        # Use settings if not explicitly provided
        if bleed is None:
            bleed = self.config.getboolean('Shoggoth', 'export_bleed', True)
        if format is None:
            format = self.config.get('Shoggoth', 'export_format', 'png')
        if quality is None:
            quality = self.config.getint('Shoggoth', 'export_quality', 95)
        if separate_versions is None:
            separate_versions = self.config.getboolean('Shoggoth', 'export_separate_versions', False)

        try:
            export_folder = Path(self.active_project.file_path).parent / f'Export of {self.active_project.name}'
            export_folder.mkdir(parents=True, exist_ok=True)

            cards = self.active_project.get_all_cards()

            # Show progress
            from PySide6.QtWidgets import QProgressDialog
            progress = QProgressDialog(
                tr("STATUS_EXPORTING"), tr("BTN_CANCEL"), 0, len(cards), self
            )
            progress.setWindowModality(Qt.WindowModal)

            # Export each card
            import threading
            threads = []
            for i, card in enumerate(cards):
                if progress.wasCanceled():
                    break

                progress.setValue(i)
                progress.setLabelText(tr("MSG_EXPORTING_CARD").format(name=card.name))

                # Export in thread
                thread = threading.Thread(
                    target=self.card_renderer.export_card_images,
                    args=(card, str(export_folder)),
                    kwargs={
                        'size': self._get_export_size(),
                        'include_backs': False,
                        'bleed': bleed,
                        'format': format,
                        'quality': quality,
                        'separate_versions': separate_versions
                    }
                )
                threads.append(thread)
                thread.start()

            # Wait for all threads
            for thread in threads:
                thread.join()

            progress.setValue(len(cards))

            QMessageBox.information(
                self,
                tr("DLG_EXPORT_COMPLETE"),
                tr("MSG_EXPORTED_CARDS").format(count=len(cards), folder=export_folder)
            )
        except Exception as e:
            QMessageBox.critical(self, tr("DLG_EXPORT_ERROR"), tr("ERR_EXPORT_CARDS").format(error=e))

    def new_card_dialog(self):
        """Show dialog to create a new card"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return

        from shoggoth.ui.dialogs import NewCardDialog
        dialog = NewCardDialog(self)
        dialog.exec()

    def show_card(self, card):
        """Display a card in the editor"""
        self._push_nav_history('card', card.id)

        # Clean up previous editor
        self.clear_editor()

        self.current_card = card
        self.current_editor = None
        self.settings['session']['last_id'] = card.id
        self.settings['session']['last_type'] = 'card'
        self._save_session()

        # Update file monitoring for this card's dependencies
        if self.card_file_monitor:
            card_files = self.card_file_monitor.get_card_file_dependencies(card)
            self.card_file_monitor.set_card_files(card_files)

        # Clear content layout
        while self.content_layout.count():
            item = self.content_layout.takeAt(0)
            if item.widget():
                item.widget().setParent(None)

        # Create a splitter for editor + preview
        card_splitter = QSplitter(Qt.Horizontal)

        # Load card editor
        editor = CardEditor(card)
        self.current_editor = editor

        # Connect data change signal to debounced preview update
        editor.data_changed.connect(self.schedule_preview_update)

        # Connect illustration mode signals
        self._connect_illustration_mode(editor)

        # Wrap editor in scroll area
        editor_scroll = QScrollArea()
        editor_scroll.setWidgetResizable(True)
        editor_scroll.setWidget(editor)

        card_splitter.addWidget(editor_scroll)

        # Show preview dock
        self.preview_dock.show()
        self.toggle_preview_action.setChecked(True)

        # Enter translation mode if this card belongs to a translation project
        if card.expansion.data.get('project'):
            editor.enter_translation_mode()

        # Set splitter sizes (60% editor, 40% would be preview but it's docked)
        card_splitter.setSizes([200, 200])

        # Add splitter to content area
        self.content_layout.addWidget(card_splitter)

        # Update preview
        self.update_card_preview()

    def _disconnect_illustration_mode(self):
        """Disconnect illustration mode signals"""
        try:
            self.card_preview.illustration_pan_changed.disconnect(self._on_illustration_pan)
            self.card_preview.illustration_scale_changed.disconnect(self._on_illustration_scale)
        except RuntimeError:
            pass  # Signals weren't connected

        # Reset illustration mode on preview
        self.card_preview.set_illustration_mode(False)

    def _connect_illustration_mode(self, editor):
        """Connect illustration mode signals between editor and preview"""
        # Disconnect any previous connections first
        self._disconnect_illustration_mode()

        # Connect preview pan/scale signals to editor
        self.card_preview.illustration_pan_changed.connect(self._on_illustration_pan)
        self.card_preview.illustration_scale_changed.connect(self._on_illustration_scale)

        # Connect editor illustration mode signals to preview
        for face_editor in [editor.front_editor, editor.back_editor]:
            if face_editor and hasattr(face_editor, 'illustration_widget'):
                widget = face_editor.illustration_widget
                widget.illustration_mode_changed.connect(self._on_illustration_mode_changed)

    def _on_illustration_mode_changed(self, enabled, side):
        """Handle illustration mode toggle from editor"""
        self.card_preview.set_illustration_mode(enabled, side)

    def _on_illustration_pan(self, side, delta_x, delta_y):
        """Handle pan changes from preview"""
        if not self.current_editor:
            return

        # Find the right face editor and illustration widget
        face_editor = self.current_editor.front_editor if side == 'front' else self.current_editor.back_editor
        if face_editor and hasattr(face_editor, 'illustration_widget'):
            face_editor.illustration_widget.update_pan(delta_x, delta_y)

    def _on_illustration_scale(self, side, delta):
        """Handle scale changes from preview"""
        if not self.current_editor:
            return

        # Find the right face editor and illustration widget
        face_editor = self.current_editor.front_editor if side == 'front' else self.current_editor.back_editor
        if face_editor and hasattr(face_editor, 'illustration_widget'):
            face_editor.illustration_widget.update_scale(delta)

    def show_encounter(self, encounter):
        """Display an encounter set in the editor"""
        self._push_nav_history('encounter', encounter.id)

        # Clear current editor
        self.clear_editor()

        # Save selection
        self.settings['session']['last_id'] = encounter.id
        self.settings['session']['last_type'] = 'encounter'
        self._save_session()

        # Hide preview for non-card views
        self.preview_dock.hide()

        # Clear content layout
        while self.content_layout.count():
            item = self.content_layout.takeAt(0)
            if item.widget():
                item.widget().setParent(None)

        # Create and show encounter set editor
        editor = EncounterSetEditor(encounter, self.card_renderer)
        editor.card_clicked.connect(self.show_card)

        # Wrap in scroll area
        scroll = QScrollArea()
        scroll.setWidgetResizable(True)
        scroll.setWidget(editor)

        self.content_layout.addWidget(scroll)

        # Update status
        self.status_bar.showMessage(tr("STATUS_EDITING_ENCOUNTER_SET").format(name=encounter.name))

    def show_encounter_set(self, encounter_set):
        """Show encounter set editor (alias for show_encounter)"""
        self.show_encounter(encounter_set)

    def show_project(self, project):
        """Display project editor"""
        self._push_nav_history('project', project.file_path)

        # Clear current editor
        self.clear_editor()

        # Hide preview for non-card views
        self.preview_dock.hide()
        self.toggle_preview_action.setChecked(False)

        # Clear content layout
        while self.content_layout.count():
            item = self.content_layout.takeAt(0)
            if item.widget():
                item.widget().setParent(None)

        # Create project editor
        from shoggoth.ui.project_editor import ProjectEditor
        editor = ProjectEditor(project, self.card_renderer)

        # Wrap in scroll area
        scroll = QScrollArea()
        scroll.setWidgetResizable(True)
        scroll.setWidget(editor)

        self.content_layout.addWidget(scroll)
        self.status_bar.showMessage(tr("STATUS_EDITING_PROJECT").format(name=project['name']))

    def show_guide(self, guide):
        """Display guide editor"""
        self._push_nav_history('guide', guide.id)

        # Clear current editor
        self.clear_editor()

        # Save selection
        self.settings['session']['last_id'] = guide.id
        self.settings['session']['last_type'] = 'guide'
        self._save_session()

        # Hide preview for non-card views
        self.preview_dock.hide()

        # Clear content layout
        while self.content_layout.count():
            item = self.content_layout.takeAt(0)
            if item.widget():
                item.widget().setParent(None)

        # Create and add guide editor
        from shoggoth.ui.guide_editor import GuideEditor
        editor = GuideEditor(guide)
        self.content_layout.addWidget(editor)

        self.current_guide = guide

    def show_locations(self, encounter_set):
        """Display location connection editor for an encounter set"""
        self._push_nav_history('locations', encounter_set.id)

        # Clear current editor
        self.clear_editor()

        # Save selection
        self.settings['session']['last_id'] = encounter_set.id
        self.settings['session']['last_type'] = 'locations'
        self._save_session()

        # Hide preview for non-card views
        self.preview_dock.hide()

        # Clear content layout
        while self.content_layout.count():
            item = self.content_layout.takeAt(0)
            if item.widget():
                item.widget().setParent(None)

        # Create and add location view
        from shoggoth.ui.location_view import LocationViewWidget
        self.location_view = LocationViewWidget(encounter_set, self.card_renderer)
        self.location_view.card_selected.connect(self.show_card)
        self.content_layout.addWidget(self.location_view)

        self.status_bar.showMessage(tr("STATUS_EDITING_LOCATIONS").format(name=encounter_set.name))

    # ── Translation management ────────────────────────────────────────────────

    def add_translation_dialog(self):
        """Prompt for a language code and create a new translation sidecar file."""
        if not self.active_project:
            return
        from PySide6.QtWidgets import QInputDialog
        lang, ok = QInputDialog.getText(
            self, "Add Translation", "Language code (e.g. dk, de, fr):"
        )
        if not ok or not lang.strip():
            return
        lang = lang.strip().lower()
        if lang in self.active_project.data.get('translations', {}):
            QMessageBox.warning(self, "Add Translation", f"Translation '{lang}' already exists.")
            return

        from pathlib import Path
        project_path = Path(self.active_project.file_path)
        translation_path = project_path.parent / f"{project_path.stem}_{lang}.json"
        data = Translation.new(project_path, lang)
        tr_obj = Translation(translation_path, data, lang)
        tr_obj.save()

        self.active_project.add_translation(lang, translation_path.name)
        self.active_project.save_all()
        self.status_bar.showMessage(f"Translation '{lang}' added: {translation_path.name}")

        # Auto-open the new translation project
        self.open_project(str(translation_path))

    def load_translation_dialog(self):
        """Show dialog to load an existing registered translation."""
        if not self.active_project:
            QMessageBox.information(self, "Load Translation", "Open a project first.")
            return
        translations = self.active_project.translations  # {lang: Path}
        if not translations:
            QMessageBox.information(self, "Load Translation",
                "This project has no registered translations.")
            return
        from PySide6.QtWidgets import QInputDialog
        choices = [f"{lang}  ({path.name})" for lang, path in translations.items()]
        choice, ok = QInputDialog.getItem(
            self, "Load Translation", "Choose a translation:", choices, 0, False)
        if not ok:
            return
        lang = choice.split("  ")[0]
        self.open_translation(str(translations[lang]))

    def open_translation(self, file_path):
        """Open a translation file and add its translated project to the tree."""
        try:
            # Avoid duplicate opens — keyed by translation file path
            for project in self.open_projects:
                if getattr(project, '_node_id_path', None) == file_path:
                    self.file_browser.set_active_project(project)
                    self.status_bar.showMessage(
                        f"Switched to {project._translation.language} translation")
                    return

            translation = Translation.load(file_path)
            project = translation.project
            project._translation = translation      # ephemeral: language reference
            project._node_id_path = file_path       # ephemeral: unique node key

            self.open_projects.append(project)
            self.active_project = project
            self.file_browser.add_project(project)
            self._save_session()
            self.status_bar.showMessage(
                f"Opened {translation.language} translation of {project['name']}")
        except Exception as e:
            QMessageBox.critical(self, "Error", f"Could not open translation:\n{e}")

    # ── Preview ───────────────────────────────────────────────────────────────

    def schedule_preview_update(self):
        """Schedule a debounced preview update (400ms delay)"""
        if not self.current_card:
            return
        # Increment version to invalidate any in-progress renders
        self.render_version += 1
        # Restart the debounce timer
        self.render_timer.start(100)

    def _start_background_render(self):
        """Start rendering in background thread"""
        if not self.current_card:
            return

        # Capture current state for the background thread
        card = self.current_card
        version = self.render_version
        bleed = 'mark' if self.config.getboolean('Shoggoth', 'show_bleed', True) else False
        renderer = self.card_renderer

        def render_task():
            try:
                front_image, back_image = renderer.get_card_textures(
                    card, {'width': 750, 'height': 1050, 'bleed': 36}, bleed=bleed
                )
                # Emit result signal (will be handled on main thread)
                self.render_result_signal.emit(version, front_image, back_image)
            except Exception as e:
                # Emit with None images to signal error
                print(f"Render error: {e}")
                self.render_result_signal.emit(version, None, None)

        thread = threading.Thread(target=render_task, daemon=True)
        thread.start()

    @Slot(int, object, object)
    def _handle_render_result(self, version, front_image, back_image):
        """Handle render result on main thread"""
        # Discard stale results
        if version != self.render_version:
            return

        if front_image is not None:
            self.card_preview.set_card_images(front_image, back_image)
        else:
            self.status_bar.showMessage(tr("ERR_RENDER_CARD"))

    def update_card_preview(self):
        """Update the card preview immediately (synchronous, for initial load)"""
        if not self.current_card:
            return

        try:
            # Get bleed setting
            bleed = False
            if self.config.getboolean('Shoggoth', 'show_bleed', True):
                bleed = 'mark'

            front_image, back_image = self.card_renderer.get_card_textures(
                self.current_card, {'width': 750, 'height': 1050, 'bleed': 36}, bleed=bleed
            )
            self.card_preview.set_card_images(front_image, back_image)
        except Exception as e:
            self.status_bar.showMessage(tr("ERR_RENDER_CARD_DETAIL").format(error=e))

    def goto_card(self, card_id):
        """Navigate to a specific card by ID"""
        if not self.active_project:
            return

        card = self.active_project.get_card(card_id)
        if card:
            self.show_card(card)
            self.select_item_in_tree(card_id)

    def refresh_tree(self):
        """Refresh the file browser tree"""
        self.file_browser.refresh()

    def update_card_in_tree(self, card_id):
        """Update a single card's display in the tree (for name/dirty changes)"""
        return self.file_browser.update_card_node(card_id)

    # --- Navigation History (browser-like back/forward) ---

    def _push_nav_history(self, nav_type, nav_id):
        """Push a navigation item to history, truncating any forward history"""
        if self._nav_navigating:
            return  # Don't record history during back/forward navigation

        nav_item = (nav_type, nav_id)

        # Don't add duplicates if we're already at this item
        if self._nav_history and self._nav_index >= 0:
            if self._nav_history[self._nav_index] == nav_item:
                return

        # Truncate forward history
        self._nav_history = self._nav_history[:self._nav_index + 1]

        # Add new item
        self._nav_history.append(nav_item)
        self._nav_index = len(self._nav_history) - 1

    def navigate_back(self):
        """Navigate to the previous item in history"""
        if self._nav_index <= 0:
            return  # Nothing to go back to

        self._nav_index -= 1
        self._navigate_to_history_item(self._nav_history[self._nav_index])

    def navigate_forward(self):
        """Navigate to the next item in history"""
        if self._nav_index >= len(self._nav_history) - 1:
            return  # Nothing to go forward to

        self._nav_index += 1
        self._navigate_to_history_item(self._nav_history[self._nav_index])

    def _navigate_to_history_item(self, nav_item):
        """Navigate to a history item without adding to history"""
        nav_type, nav_id = nav_item
        self._nav_navigating = True
        try:
            if nav_type == 'card':
                card = self.active_project.get_card(nav_id)
                if card:
                    self.show_card(card)
                    self.select_item_in_tree(nav_id)
            elif nav_type == 'encounter':
                encounter = self.active_project.get_encounter_set(nav_id)
                if encounter:
                    self.show_encounter(encounter)
                    self.select_item_in_tree(nav_id)
            elif nav_type == 'project':
                self.show_project(self.active_project)
            elif nav_type == 'guide':
                guide = self.active_project.get_guide(nav_id)
                if guide:
                    self.show_guide(guide)
                    self.select_item_in_tree(nav_id)
            elif nav_type == 'locations':
                encounter = self.active_project.get_encounter_set(nav_id)
                if encounter:
                    self.show_locations(encounter)
        finally:
            self._nav_navigating = False

    def mousePressEvent(self, event):
        """Handle mouse button presses for back/forward navigation"""
        from PySide6.QtCore import Qt
        if event.button() == Qt.BackButton:
            self.navigate_back()
            event.accept()
        elif event.button() == Qt.ForwardButton:
            self.navigate_forward()
            event.accept()
        else:
            super().mousePressEvent(event)

    def export_current(self, bleed=None, format=None, quality=None, separate_versions=None):
        """Export the current card using settings from preferences"""
        if not self.current_card:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_CARD_SELECTED"))
            return

        # Use settings if not explicitly provided
        if bleed is None:
            bleed = self.config.getboolean('Shoggoth', 'export_bleed', True)
        if format is None:
            format = self.config.get('Shoggoth', 'export_format', 'png')
        if quality is None:
            quality = self.config.getint('Shoggoth', 'export_quality', 95)
        if separate_versions is None:
            separate_versions = self.config.getboolean('Shoggoth', 'export_separate_versions', False)

        try:
            export_folder = Path(self.active_project.file_path).parent / f'Export of {self.active_project.name}'
            export_folder.mkdir(parents=True, exist_ok=True)

            # Export card
            self.card_renderer.export_card_images(
                self.current_card,
                str(export_folder),
                size=self._get_export_size(),
                include_backs=False,
                bleed=bleed,
                format=format,
                quality=quality,
                separate_versions=separate_versions
            )

            QMessageBox.information(
                self,
                tr("DLG_EXPORT_COMPLETE"),
                tr("MSG_CARD_EXPORTED").format(folder=export_folder)
            )
        except Exception as e:
            QMessageBox.critical(self, tr("DLG_EXPORT_ERROR"), tr("ERR_EXPORT_CARD").format(error=e))

    def on_file_changed(self, file_path):
        """Called from file monitor (background thread) - emit signal for main thread handling"""
        # Emit signal to handle on main thread (Qt requires UI updates on main thread)
        self.file_changed_signal.emit(file_path)

    @Slot(str)
    def _handle_file_changed(self, file_path):
        """Handle file system changes on main thread - refresh preview when relevant files change"""
        if self.current_card:
            # Invalidate the renderer cache for the changed file
            self.card_renderer.invalidate_cache(file_path)

            # Reload fallback data (for template/defaults changes)
            self.current_card.reload_fallback()

            # Update the preview (increment version and start background render immediately)
            self.render_version += 1
            self._start_background_render()

    def show_about(self):
        """Show about dialog"""
        # Get version from package metadata
        try:
            from importlib.metadata import version
            app_version = version("shoggoth")
        except Exception:
            app_version = "unknown"

        # URLs for links
        urls = {
            'contrib': 'https://github.com/tokeeto/shoggoth',
            'patreon': 'https://www.patreon.com/tokeeto',
            'tips': 'https://ko-fi.com/tokeeto',
        }

        about_html = f"""
        <div style="text-align: center;">
            <h1 style="font-size: 32pt; margin-bottom: 5px;">Shoggoth</h1>
            <p style="font-size: 14pt; color: #666;">{tr("ABOUT_VERSION").format(version=app_version)}</p>
        </div>
        <hr>
        <p>{tr("ABOUT_CREATED_BY")}</p>
        <p>{tr("ABOUT_SUPPORT_TEXT").format(
            contributing=f'<a href="{urls["contrib"]}">{tr("ABOUT_CONTRIBUTING")}</a>',
            donating=f'<a href="{urls["patreon"]}">{tr("ABOUT_DONATING")}</a>',
            tipping=f'<a href="{urls["tips"]}">{tr("ABOUT_TIPPING")}</a>'
        )}</p>
        <p>{tr("ABOUT_IMAGES_CREDIT")}</p>
        <p>{tr("ABOUT_THANKS_SE")}</p>
        <p>{tr("ABOUT_SPECIAL_THANKS")}</p>
        """

        dialog = QDialog(self)
        dialog.setWindowTitle(tr("DLG_ABOUT_TITLE"))
        dialog.setMinimumSize(450, 350)

        layout = QVBoxLayout()

        # Use QTextBrowser for clickable links
        text_browser = QTextBrowser()
        text_browser.setOpenExternalLinks(True)
        text_browser.setHtml(about_html)
        text_browser.setStyleSheet("background: transparent; border: none;")
        layout.addWidget(text_browser)

        # OK button
        button_box = QDialogButtonBox(QDialogButtonBox.Ok)
        button_box.accepted.connect(dialog.accept)
        layout.addWidget(button_box)

        dialog.setLayout(layout)
        dialog.exec()

    def open_asset_location(self):
        """Open the asset folder in the system file explorer"""
        QDesktopServices.openUrl(QUrl.fromLocalFile(str(asset_dir)))

    def closeEvent(self, event):
        """Handle window close event - check for unsaved changes"""
        if self.card_file_monitor:
            self.card_file_monitor.stop()
        self.save_settings()

        if self.has_unsaved_changes():
            msg_box = QMessageBox(self)
            msg_box.setWindowTitle(tr("DLG_UNSAVED_CHANGES"))
            msg_box.setText(tr("CONFIRM_SAVE_BEFORE_EXIT"))
            msg_box.setIcon(QMessageBox.Question)
            save_btn = msg_box.addButton(tr("DLG_SAVE"), QMessageBox.AcceptRole)
            discard_btn = msg_box.addButton(tr("DLG_DISCARD"), QMessageBox.DestructiveRole)
            cancel_btn = msg_box.addButton(tr("DLG_CANCEL"), QMessageBox.RejectRole)
            msg_box.setDefaultButton(save_btn)
            msg_box.exec()

            if msg_box.clickedButton() == save_btn:
                self.save_changes()
                event.accept()
            elif msg_box.clickedButton() == discard_btn:
                event.accept()
            else:  # Cancel
                event.ignore()
        else:
            event.accept()

    def has_unsaved_changes(self):
        """Check if there are any unsaved changes in the project"""
        if not self.active_project:
            return False

        for card in self.active_project.get_all_cards():
            if hasattr(card, 'dirty') and card.dirty:
                return True
            if hasattr(card, 'front') and hasattr(card.front, 'dirty') and card.front.dirty:
                return True
            if hasattr(card, 'back') and hasattr(card.back, 'dirty') and card.back.dirty:
                return True

        return False

    # ==================== PROJECT MENU ACTIONS ====================

    def auto_enumerate(self):
        """Add a guide to the project"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        self.active_project.assign_card_numbers()
        self.status_bar.showMessage(tr("STATUS_PROJECT_ENUMERATED"))

    def add_guide(self):
        """Add a guide to the project"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        self.active_project.add_guide()
        self.file_browser.refresh()
        self.status_bar.showMessage(tr("STATUS_GUIDE_ADDED"))

    def add_scenario_template(self):
        """Add a scenario template"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        name, ok = self.get_text_input(tr("DLG_SCENARIO_NAME"), tr("MSG_ENTER_SCENARIO"), tr("PLACEHOLDER_SCENARIO"))
        if ok and name:
            self.active_project.create_scenario(name)
            self.file_browser.refresh()
            self.status_bar.showMessage(tr("STATUS_SCENARIO_CREATED").format(name=name))

    def add_campaign_template(self):
        """Add a campaign template"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        self.active_project.create_campaign()
        self.file_browser.refresh()
        self.status_bar.showMessage(tr("STATUS_CAMPAIGN_CREATED"))

    def add_investigator_template(self):
        """Add an investigator template"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        name, ok = self.get_text_input(tr("DLG_INVESTIGATOR_NAME"), tr("MSG_ENTER_INVESTIGATOR"), tr("PLACEHOLDER_ROLAN"))
        if ok and name:
            self.active_project.add_investigator_set(name)
            self.file_browser.refresh()
            self.status_bar.showMessage(tr("STATUS_INVESTIGATOR_CREATED").format(name=name))

    def add_investigator_expansion_template(self):
        """Add an investigator expansion template"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        self.active_project.create_player_expansion()
        self.file_browser.refresh()
        self.status_bar.showMessage(tr("STATUS_EXPANSION_CREATED"))

    # ==================== EXPORT MENU ACTIONS ====================

    def export_card_to_pdf(self):
        """Export current card to PDF"""
        if not self.current_card:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_CARD_SELECTED"))
            return
        # TODO: Implement PDF export
        QMessageBox.information(self, tr("DLG_TODO"), tr("MSG_PDF_NOT_IMPLEMENTED"))

    def export_cards_to_pdf(self):
        """Export all cards to PDF"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        # TODO: Implement PDF export
        QMessageBox.information(self, tr("DLG_TODO"), tr("MSG_PDF_NOT_IMPLEMENTED"))

    def export_project_to_pdf(self):
        """Export entire project to PDF"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        # TODO: Implement PDF export
        QMessageBox.information(self, tr("DLG_TODO"), tr("MSG_PDF_NOT_IMPLEMENTED"))

    def export_card_to_tts(self):
        """Export card to Tabletop Simulator"""
        if not self.current_card:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_CARD_SELECTED"))
            return
        try:
            from shoggoth import tts_lib
            tts_lib.export_card(self.current_card)
            self.status_bar.showMessage(tr("STATUS_TTS_CARD_EXPORTED"))
        except Exception as e:
            QMessageBox.critical(self, tr("DLG_ERROR"), tr("ERR_EXPORT_TTS").format(error=e))

    def export_campaign_to_tts(self):
        """Export campaign cards to Tabletop Simulator"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        try:
            from shoggoth import tts_lib
            tts_lib.export_campaign(self.active_project)
            self.status_bar.showMessage(tr("STATUS_TTS_CAMPAIGN_EXPORTED"))
        except Exception as e:
            QMessageBox.critical(self, tr("DLG_ERROR"), tr("ERR_EXPORT_TTS").format(error=e))

    def export_player_to_tts(self):
        """Export player cards to Tabletop Simulator"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return
        try:
            from shoggoth import tts_lib
            tts_lib.export_player_cards(self.active_project.player_cards)
            self.status_bar.showMessage(tr("STATUS_TTS_PLAYER_EXPORTED"))
        except Exception as e:
            QMessageBox.critical(self, tr("DLG_ERROR"), tr("ERR_EXPORT_TTS").format(error=e))

    # ==================== TOOLS MENU ACTIONS ====================

    def convert_strange_eons(self):
        """Convert a Strange Eons project"""
        pass

    # ==================== FILE MENU ACTIONS ====================

    def gather_images(self, update=False):
        """Gather all images from the project"""
        if not self.active_project:
            QMessageBox.warning(self, tr("DLG_ERROR"), tr("MSG_NO_PROJECT_OPEN"))
            return

        try:
            self.active_project.gather_images(update=update)
            action = tr("ACTION_GATHERED_UPDATED") if update else tr("ACTION_GATHERED")
            self.status_bar.showMessage(tr("STATUS_IMAGES_ACTION").format(action=action))
            QMessageBox.information(self, tr("DLG_SUCCESS"), tr("MSG_IMAGES_SUCCESS").format(action=action))
        except Exception as e:
            QMessageBox.critical(self, tr("DLG_ERROR"), tr("ERR_GATHER_IMAGES").format(error=e))

    def open_settings(self):
        """Open settings dialog"""
        from shoggoth.settings import SettingsDialog
        dialog = SettingsDialog(self.config, self)
        dialog.exec()

    # ==================== HELP MENU ACTIONS ====================

    def show_text_options(self):
        """Show text formatting options"""
        help_text = self.card_renderer.rich_text.get_help_text()
        msg = QMessageBox(self)
        msg.setWindowTitle(tr("DLG_TEXT_OPTIONS"))
        msg.setText(tr("MSG_RICH_TEXT_OPTIONS"))
        msg.setDetailedText(help_text)
        msg.exec()

    # ==================== UTILITY METHODS ====================

    def get_text_input(self, title, label, default=""):
        """Show a text input dialog"""
        from PySide6.QtWidgets import QInputDialog
        text, ok = QInputDialog.getText(self, title, label, text=default)
        return text, ok



"""
Go to Card dialog with fuzzy search
"""
from PySide6.QtWidgets import (
    QDialog, QVBoxLayout, QLineEdit, QListWidget, QListWidgetItem,
    QLabel, QWidget, QHBoxLayout
)
from PySide6.QtCore import Qt, Signal, QTimer
from PySide6.QtGui import QFont
from shoggoth.i18n import tr


def fuzzy_match(pattern, text):
    """
    Fuzzy match pattern against text.
    Returns (match_score, matched_indices) or (0, []) if no match.
    
    Higher score = better match
    """
    pattern = pattern.lower()
    text = text.lower()
    
    if not pattern:
        return (0, [])
    
    # Exact match gets highest score
    if pattern in text:
        start = text.index(pattern)
        indices = list(range(start, start + len(pattern)))
        return (1000 + len(pattern), indices)
    
    # Fuzzy match
    pattern_idx = 0
    text_idx = 0
    matched_indices = []
    score = 0
    consecutive = 0
    
    while pattern_idx < len(pattern) and text_idx < len(text):
        if pattern[pattern_idx] == text[text_idx]:
            matched_indices.append(text_idx)
            pattern_idx += 1
            consecutive += 1
            score += 1 + consecutive  # Bonus for consecutive matches
        else:
            consecutive = 0
        text_idx += 1
    
    # All pattern characters must be matched
    if pattern_idx != len(pattern):
        return (0, [])
    
    # Bonus for matches at word boundaries
    for idx in matched_indices:
        if idx == 0 or text[idx-1] in (' ', '_', '-', '/'):
            score += 5
    
    return (score, matched_indices)


class CardListItem(QWidget):
    """Custom widget for displaying card in the list"""
    
    def __init__(self, card, path, search_term=""):
        super().__init__()
        self.card = card
        self.path = path
        
        layout = QVBoxLayout()
        layout.setContentsMargins(5, 3, 5, 3)
        layout.setSpacing(2)
        
        # Card name (larger, bold)
        self.name_label = QLabel()
        name_font = QFont()
        name_font.setPointSize(11)
        name_font.setBold(True)
        self.name_label.setFont(name_font)
        
        # Highlight matching characters
        self.set_name_with_highlight(card.name, search_term)
        
        layout.addWidget(self.name_label)
        
        # Path (smaller, gray)
        path_label = QLabel(path)
        path_font = QFont()
        path_font.setPointSize(9)
        path_label.setFont(path_font)
        path_label.setStyleSheet("color: #666666;")
        layout.addWidget(path_label)
        
        self.setLayout(layout)
    
    def set_name_with_highlight(self, name, search_term):
        """Set the name with highlighted matching characters"""
        if not search_term:
            self.name_label.setText(name)
            return
        
        score, indices = fuzzy_match(search_term, name)
        if score == 0:
            self.name_label.setText(name)
            return
        
        # Build HTML with highlighted characters
        html = ""
        for i, char in enumerate(name):
            if i in indices:
                html += f'<span style="background-color: #ffeb3b; color: #000;">{char}</span>'
            else:
                html += char
        
        self.name_label.setText(html)


class GotoCardDialog(QDialog):
    """Dialog for quickly navigating to cards with fuzzy search"""
    
    card_selected = Signal(object)  # Emits the selected card
    
    def __init__(self, project, parent=None):
        super().__init__(parent)
        self.project = project
        self.all_cards = []
        self.filtered_cards = []
        
        self.setWindowTitle(tr("DLG_GOTO_CARD"))
        self.setModal(True)
        self.resize(600, 400)
        
        # Setup UI
        self.setup_ui()
        
        # Build card list
        self.build_card_list()
        
        # Show all cards initially
        self.update_results("")
        
        # Focus the search box
        self.search_input.setFocus()
    
    def setup_ui(self):
        """Setup the dialog UI"""
        layout = QVBoxLayout()
        
        # Search input
        self.search_input = QLineEdit()
        self.search_input.setPlaceholderText(tr("MSG_TYPE_TO_SEARCH"))
        self.search_input.textChanged.connect(self.on_search_changed)
        
        # Make search input larger
        search_font = QFont()
        search_font.setPointSize(12)
        self.search_input.setFont(search_font)
        self.search_input.setMinimumHeight(35)
        
        layout.addWidget(self.search_input)
        
        # Results count label
        self.count_label = QLabel()
        self.count_label.setStyleSheet("color: #666666; font-size: 10pt;")
        layout.addWidget(self.count_label)
        
        # Results list
        self.results_list = QListWidget()
        self.results_list.itemDoubleClicked.connect(self.on_item_double_clicked)
        layout.addWidget(self.results_list)
        
        # Instructions
        instructions = QLabel(
            tr("MSG_GOTO_INSTRUCTIONS")
        )
        instructions.setStyleSheet("color: #999999; font-size: 9pt; padding: 5px;")
        instructions.setAlignment(Qt.AlignCenter)
        layout.addWidget(instructions)
        
        self.setLayout(layout)
    
    def build_card_list(self):
        """Build the list of all cards with their paths"""
        if not self.project:
            return
        
        # Process encounter sets (campaign cards)
        for encounter_set in self.project.encounter_sets:
            for card in encounter_set.cards:
                # Determine subcategory
                if card.front.get('type') == 'location':
                    subcategory = "Locations"
                elif card.back.get('type') == 'encounter':
                    subcategory = "Encounter"
                else:
                    subcategory = "Story"
                
                path = f"Campaign Cards / {encounter_set.name} / {subcategory}"
                self.all_cards.append((card, path))
        
        # Process player cards
        for card in self.project.player_cards:
            # Check if it's part of an investigator set
            if investigator := card.data.get('investigator'):
                path = f"Player Cards / Investigators / {investigator}"
            else:
                # Get card class
                card_class = card.get_class()
                if card_class:
                    class_name = {
                        'guardian': 'Guardian',
                        'seeker': 'Seeker',
                        'rogue': 'Rogue',
                        'mystic': 'Mystic',
                        'survivor': 'Survivor',
                        'neutral': 'Neutral',
                        'multi': 'Multi-class'
                    }.get(card_class, 'Other')
                    path = f"Player Cards / {class_name}"
                else:
                    path = "Player Cards / Other"
            
            self.all_cards.append((card, path))
        
        # Process guides
        for guide in self.project.guides:
            path = "Guides"
            # Create a pseudo-card object for guides
            class GuideWrapper:
                def __init__(self, guide):
                    self.guide = guide
                    self.name = guide.name
                    self.id = guide.id
                    self.is_guide = True
            
            self.all_cards.append((GuideWrapper(guide), path))
    
    def on_search_changed(self, text):
        """Handle search text changes"""
        # Debounce search for performance
        if hasattr(self, 'search_timer'):
            self.search_timer.stop()
        
        self.search_timer = QTimer()
        self.search_timer.setSingleShot(True)
        self.search_timer.timeout.connect(lambda: self.update_results(text))
        self.search_timer.start(100)  # 100ms debounce
    
    def update_results(self, search_term):
        """Update the results list based on search term"""
        self.results_list.clear()
        
        if not search_term:
            # No search term - show all cards
            results = [(card, path, 0) for card, path in self.all_cards]
        else:
            # Fuzzy search
            results = []
            for card, path in self.all_cards:
                score, _ = fuzzy_match(search_term, card.name)
                if score > 0:
                    results.append((card, path, score))
            
            # Sort by score (highest first)
            results.sort(key=lambda x: x[2], reverse=True)
        
        # Limit results to top 100 for performance
        results = results[:100]
        
        # Update count label
        total = len([c for c, p in self.all_cards])
        shown = len(results)
        if search_term:
            self.count_label.setText(tr("MSG_SHOWING_CARDS").format(shown=shown, total=total))
        else:
            self.count_label.setText(tr("MSG_TOTAL_CARDS").format(total=total))
        
        # Add results to list
        for card, path, score in results:
            item = QListWidgetItem(self.results_list)
            widget = CardListItem(card, path, search_term)
            item.setSizeHint(widget.sizeHint())
            item.setData(Qt.UserRole, card)  # Store card object
            self.results_list.addItem(item)
            self.results_list.setItemWidget(item, widget)
        
        # Select first item
        if self.results_list.count() > 0:
            self.results_list.setCurrentRow(0)
    
    def keyPressEvent(self, event):
        """Handle key presses"""
        if event.key() == Qt.Key_Escape:
            self.reject()
        elif event.key() in (Qt.Key_Return, Qt.Key_Enter):
            self.select_current_item()
        elif event.key() in (Qt.Key_Down, Qt.Key_Up):
            # Let the list widget handle it
            self.results_list.setFocus()
            self.results_list.keyPressEvent(event)
            self.search_input.setFocus()
        else:
            super().keyPressEvent(event)
    
    def on_item_double_clicked(self, item):
        """Handle double-click on item"""
        self.select_current_item()
    
    def select_current_item(self):
        """Select the currently highlighted item"""
        current_item = self.results_list.currentItem()
        if current_item:
            card = current_item.data(Qt.UserRole)
            self.card_selected.emit(card)
            self.accept()
    
    def showEvent(self, event):
        """Handle show event"""
        super().showEvent(event)
        # Focus search input when dialog is shown
        self.search_input.setFocus()
        self.search_input.selectAll()


"""
Export helpers - shared utilities for exporting cards to various formats
"""

# Mapping from Shoggoth face types to expected front/back pairs
# Format: front_type -> (export_type, expected_back_type)
CARD_TYPE_INFO = {
    # Player cards - have "player" backs
    'asset': {
        'export_type': 'asset',
        'expected_back': 'player',
        'faction': 'from_classes',  # Derive from card's classes
    },
    'event': {
        'export_type': 'event',
        'expected_back': 'player',
        'faction': 'from_classes',
    },
    'skill': {
        'export_type': 'skill',
        'expected_back': 'player',
        'faction': 'from_classes',
    },

    # Customizable cards - have "customizable_back" backs
    'customizable': {
        'export_type': 'asset',  # Most customizables are assets
        'expected_back': 'customizable_back',
        'faction': 'from_classes',
    },

    # Investigator cards - have specific front/back
    'investigator': {
        'export_type': 'investigator',
        'expected_back': 'investigator_back',
        'faction': 'from_classes',
        'double_sided': True,
    },

    # Encounter cards - have "encounter" backs
    'enemy': {
        'export_type': 'enemy',
        'expected_back': 'encounter',
        'faction': 'mythos',
    },
    'treachery': {
        'export_type': 'treachery',
        'expected_back': 'encounter',
        'faction': 'mythos',
    },

    # Location cards - have specific front/back
    'location': {
        'export_type': 'location',
        'expected_back': 'location_back',
        'faction': 'mythos',
        'double_sided': True,
    },

    # Act cards - have specific front/back
    'act': {
        'export_type': 'act',
        'expected_back': 'act_back',
        'faction': 'mythos',
        'double_sided': True,
    },

    # Agenda cards - have specific front/back
    'agenda': {
        'export_type': 'agenda',
        'expected_back': 'agenda_back',
        'faction': 'mythos',
        'double_sided': True,
    },

    # Scenario/Chaos reference cards - same type on both sides
    'chaos': {
        'export_type': 'scenario',
        'expected_back': 'chaos',
        'faction': 'mythos',
        'double_sided': True,
    },

    # Story cards
    'story': {
        'export_type': 'story',
        'expected_back': 'story',  # Or could be encounter
        'faction': 'mythos',
    },
}

# Mapping from Shoggoth class names to arkham.build faction codes
CLASS_TO_FACTION = {
    'guardian': 'guardian',
    'seeker': 'seeker',
    'rogue': 'rogue',
    'mystic': 'mystic',
    'survivor': 'survivor',
    'neutral': 'neutral',
}


def get_card_export_type(card):
    """
    Determine the export type for a card based on its front face type.

    Returns a dict with:
        - type_code: The export type code (e.g., 'asset', 'enemy', 'location')
        - faction_code: The primary faction code
        - faction2_code: Secondary faction (for multi-class)
        - faction3_code: Tertiary faction (for multi-class)
        - double_sided: Whether the card has meaningful content on both sides
        - is_encounter: Whether this is an encounter card (vs player card)
    """
    front_type = card.front.get('type')

    if front_type not in CARD_TYPE_INFO:
        return {
            'type_code': 'unknown',
            'faction_code': 'neutral',
            'faction2_code': None,
            'faction3_code': None,
            'double_sided': False,
            'is_encounter': False,
        }

    info = CARD_TYPE_INFO[front_type]

    # Determine faction(s)
    faction_code = 'neutral'
    faction2_code = None
    faction3_code = None

    if info['faction'] == 'from_classes':
        classes = card.front.get('classes', [])
        if classes:
            if len(classes) >= 1:
                faction_code = CLASS_TO_FACTION.get(classes[0], 'neutral')
            if len(classes) >= 2:
                faction2_code = CLASS_TO_FACTION.get(classes[1], None)
            if len(classes) >= 3:
                faction3_code = CLASS_TO_FACTION.get(classes[2], None)
    else:
        faction_code = info['faction']

    return {
        'type_code': info['export_type'],
        'faction_code': faction_code,
        'faction2_code': faction2_code,
        'faction3_code': faction3_code,
        'double_sided': info.get('double_sided', False),
        'is_encounter': info['faction'] == 'mythos',
    }


def is_player_card(card):
    """Check if a card is a player card (has player back)"""
    front_type = card.front.get('type')
    if front_type in CARD_TYPE_INFO:
        info = CARD_TYPE_INFO[front_type]
        return info['expected_back'] in ('player', 'customizable_back')
    return False


def is_investigator_card(card):
    """Check if a card is an investigator card"""
    return card.front.get('type') == 'investigator'


def is_encounter_card(card):
    """Check if a card is an encounter card"""
    front_type = card.front.get('type')
    if front_type in CARD_TYPE_INFO:
        return CARD_TYPE_INFO[front_type]['faction'] == 'mythos'
    return False


def get_skill_icons(face):
    """
    Extract skill icons from a face.

    Returns a dict with:
        - skill_willpower: int
        - skill_intellect: int
        - skill_combat: int
        - skill_agility: int
        - skill_wild: int
    """
    icons = face.get('icons', [])

    counts = {
        'skill_willpower': 0,
        'skill_intellect': 0,
        'skill_combat': 0,
        'skill_agility': 0,
        'skill_wild': 0,
    }

    icon_mapping = {
        'willpower': 'skill_willpower',
        'intellect': 'skill_intellect',
        'combat': 'skill_combat',
        'agility': 'skill_agility',
        'wild': 'skill_wild',
    }

    for icon in icons:
        if icon in icon_mapping:
            counts[icon_mapping[icon]] += 1

    return counts


def parse_slot(face):
    """
    Parse slot information from a face.

    Returns the slot string for arkham.build format.
    """
    slots = face.get('slots', [])
    slot = face.get('slot')

    if slots:
        return ', '.join(slots)
    elif slot:
        return slot
    return None



#!/usr/bin/env python3
"""
Simple run script for Shoggoth PySide6 version
"""
import sys
from pathlib import Path

# Add the parent directory to Python path so we can import shoggoth modules
parent_dir = Path(__file__).parent.parent
sys.path.insert(0, str(parent_dir))

# Now we can import and run the Qt version
from shoggoth.ui.app import main

if __name__ == "__main__":
    main()


"""Translation editor widget for Shoggoth.

Shows original field values (greyed, read-only) above editable translation
fields.  Changes are persisted to the Translation object immediately.
"""
from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QLabel, QLineEdit, QPlainTextEdit,
    QGroupBox, QFrame,
)
from PySide6.QtCore import Qt

from shoggoth.i18n import tr


# Face-level fields that are worth translating: (field_name, display_label, multiline)
FACE_TEXT_FIELDS = [
    ('title',       'Title',    False),
    ('subtitle',    'Subtitle', False),
    ('text',        'Text',     True),
    ('flavor_text', 'Flavor',   True),
]


class TranslationFieldPair(QWidget):
    """Shows a read-only original value above an editable translation field."""

    def __init__(self, label, original, translated='', multiline=False, parent=None):
        super().__init__(parent)
        self.multiline = multiline

        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 4)
        layout.setSpacing(2)

        layout.addWidget(QLabel(f'<b>{label}</b>'))

        # Original value — greyed, read-only
        if multiline:
            orig = QPlainTextEdit()
            orig.setPlainText(original or '')
            orig.setMaximumHeight(70)
        else:
            orig = QLineEdit()
            orig.setText(original or '')
        orig.setReadOnly(True)
        orig.setStyleSheet('color: grey;')
        layout.addWidget(orig)

        # Editable translation input
        if multiline:
            self.input = QPlainTextEdit()
            self.input.setPlainText(translated or '')
            self.input.setMaximumHeight(100)
        else:
            self.input = QLineEdit()
            self.input.setText(translated or '')
        layout.addWidget(self.input)

        sep = QFrame()
        sep.setFrameShape(QFrame.HLine)
        sep.setFrameShadow(QFrame.Sunken)
        layout.addWidget(sep)

        self.setLayout(layout)

    def value(self):
        if self.multiline:
            return self.input.toPlainText()
        return self.input.text()

    def connect_changed(self, slot):
        if self.multiline:
            self.input.textChanged.connect(slot)
        else:
            self.input.textChanged.connect(slot)


class TranslationEditor(QWidget):
    """Editor panel for translating a single card.

    Displays original field values (greyed, read-only) above editable
    translation fields.  Changes are written to the Translation object and
    persisted to disk immediately on each keystroke.
    """

    def __init__(self, card, translation, parent=None):
        super().__init__(parent)
        self.card = card
        self.translation = translation
        self._fields = {}   # key -> TranslationFieldPair

        overlay = translation.get_overlay(card.id) or {}
        front_overlay = overlay.get('front', {})
        back_overlay = overlay.get('back', {})

        layout = QVBoxLayout()
        layout.setContentsMargins(8, 8, 8, 8)
        layout.setSpacing(8)

        # ── Card-level fields ───────────────────────────────────────────────
        card_group = QGroupBox('Card')
        card_layout = QVBoxLayout()
        card_layout.setContentsMargins(8, 8, 8, 8)

        w = TranslationFieldPair('Name', card.data.get('name', ''), overlay.get('name', ''))
        w.connect_changed(self._save)
        self._fields['name'] = w
        card_layout.addWidget(w)

        card_group.setLayout(card_layout)
        layout.addWidget(card_group)

        # ── Front face fields ───────────────────────────────────────────────
        front_group = QGroupBox(tr('TAB_FRONT'))
        front_layout = QVBoxLayout()
        front_layout.setContentsMargins(8, 8, 8, 8)

        for field, label, multiline in FACE_TEXT_FIELDS:
            original = card.front.get(field, '')
            if not original:
                continue
            w = TranslationFieldPair(label, original, front_overlay.get(field, ''), multiline)
            w.connect_changed(self._save)
            self._fields[f'front.{field}'] = w
            front_layout.addWidget(w)

        front_group.setLayout(front_layout)
        layout.addWidget(front_group)

        # ── Back face fields ────────────────────────────────────────────────
        back_group = QGroupBox(tr('TAB_BACK'))
        back_layout = QVBoxLayout()
        back_layout.setContentsMargins(8, 8, 8, 8)

        for field, label, multiline in FACE_TEXT_FIELDS:
            original = card.back.get(field, '')
            if not original:
                continue
            w = TranslationFieldPair(label, original, back_overlay.get(field, ''), multiline)
            w.connect_changed(self._save)
            self._fields[f'back.{field}'] = w
            back_layout.addWidget(w)

        back_group.setLayout(back_layout)
        layout.addWidget(back_group)

        layout.addStretch()
        self.setLayout(layout)

    def _save(self):
        """Rebuild overlay from all current field values and persist to disk."""
        overlay = {}
        for key, widget in self._fields.items():
            val = widget.value()
            if not val:
                continue
            if '.' in key:
                face, field = key.split('.', 1)
                overlay.setdefault(face, {})[field] = val
            else:
                overlay[key] = val

        self.translation.set_overlay(self.card.id, overlay)
        self.translation.save()



"""
Improved card preview widget with zoom, pan, and tabs
"""
from shoggoth.i18n import tr
from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QTabWidget, QLabel, QScrollArea,
    QSizePolicy
)
from PySide6.QtCore import Qt, QPoint, QSize, Signal
from PySide6.QtGui import QPixmap, QImage, QWheelEvent, QMouseEvent, QPainter


class ZoomableImageLabel(QLabel):
    """A label that displays an image with zoom and pan capabilities"""

    # Signals for illustration mode
    illustration_pan_changed = Signal(int, int)  # delta_x, delta_y
    illustration_scale_changed = Signal(float)   # delta scale

    def __init__(self):
        super().__init__()
        self.setAlignment(Qt.AlignCenter)
        self.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored)
        self.setScaledContents(False)  # We'll handle scaling ourselves

        # Zoom and pan state
        self.zoom_factor = 1.0
        self.min_zoom = 0.1
        self.max_zoom = 5.0
        self.zoom_step = 0.1

        # For panning
        self.panning = False
        self.pan_start_pos = QPoint()
        self.current_offset = QPoint(0, 0)

        # Original pixmap
        self.original_pixmap = None

        # Illustration mode
        self.illustration_mode = False
        self.card_scale_factor = 1.0  # Scale between original card size and displayed size

        # Enable mouse tracking for panning
        self.setMouseTracking(True)

    def setPixmap(self, pixmap):
        """Set the pixmap and reset zoom/pan"""
        self.original_pixmap = pixmap
        self.zoom_factor = 1.0
        self.current_offset = QPoint(0, 0)
        self.update_display()

    def update_display(self):
        """Update the displayed image with current zoom and pan"""
        if not self.original_pixmap:
            super().setPixmap(QPixmap())
            return

        # Calculate scaled size preserving aspect ratio
        original_size = self.original_pixmap.size()
        available_size = self.size()

        # Fit to widget size first
        scaled_size = original_size.scaled(
            available_size,
            Qt.KeepAspectRatio
        )

        # Calculate the base scale factor (before zoom)
        if original_size.width() > 0:
            self.card_scale_factor = scaled_size.width() / original_size.width()
        else:
            self.card_scale_factor = 1.0

        # Apply zoom
        final_width = int(scaled_size.width() * self.zoom_factor)
        final_height = int(scaled_size.height() * self.zoom_factor)

        # Scale the pixmap
        scaled_pixmap = self.original_pixmap.scaled(
            final_width,
            final_height,
            Qt.KeepAspectRatio,
            Qt.SmoothTransformation
        )

        super().setPixmap(scaled_pixmap)

    def set_illustration_mode(self, enabled):
        """Enable or disable illustration positioning mode"""
        self.illustration_mode = enabled
        if enabled:
            self.setCursor(Qt.CrossCursor)
        else:
            self.setCursor(Qt.ArrowCursor)

    def wheelEvent(self, event: QWheelEvent):
        """Handle mouse wheel for zooming or illustration scaling"""
        if not self.original_pixmap:
            return

        # Get wheel delta
        delta = event.angleDelta().y()

        if self.illustration_mode:
            # In illustration mode, adjust illustration scale
            scale_delta = 0.02 if delta > 0 else -0.02
            self.illustration_scale_changed.emit(scale_delta)
        else:
            # Normal zoom behavior
            if delta > 0:
                new_zoom = self.zoom_factor + self.zoom_step
            else:
                new_zoom = self.zoom_factor - self.zoom_step

            # Clamp zoom factor
            new_zoom = max(self.min_zoom, min(self.max_zoom, new_zoom))

            if new_zoom != self.zoom_factor:
                self.zoom_factor = new_zoom
                self.update_display()

    def mousePressEvent(self, event: QMouseEvent):
        """Handle mouse press for panning"""
        if self.illustration_mode and event.button() == Qt.LeftButton:
            # Illustration mode - start dragging
            self.panning = True
            self.pan_start_pos = event.pos()
            self.setCursor(Qt.ClosedHandCursor)
        elif event.button() == Qt.MiddleButton or (event.button() == Qt.LeftButton and event.modifiers() & Qt.ControlModifier):
            self.panning = True
            self.pan_start_pos = event.pos()
            self.setCursor(Qt.ClosedHandCursor)

    def mouseReleaseEvent(self, event: QMouseEvent):
        """Handle mouse release"""
        if event.button() == Qt.MiddleButton or event.button() == Qt.LeftButton:
            self.panning = False
            if self.illustration_mode:
                self.setCursor(Qt.CrossCursor)
            else:
                self.setCursor(Qt.ArrowCursor)

    def mouseMoveEvent(self, event: QMouseEvent):
        """Handle mouse move for panning"""
        if self.panning:
            delta = event.pos() - self.pan_start_pos
            self.pan_start_pos = event.pos()

            if self.illustration_mode:
                # Convert screen delta to card pixel delta
                # Account for zoom and display scaling
                scale = self.zoom_factor * self.card_scale_factor
                if scale > 0:
                    card_delta_x = int(delta.x() / scale)
                    card_delta_y = int(delta.y() / scale)
                    if card_delta_x != 0 or card_delta_y != 0:
                        self.illustration_pan_changed.emit(card_delta_x, card_delta_y)
            else:
                self.current_offset += delta
                # Note: For true panning, we'd need to use QGraphicsView
                # For now, zoom is the main feature

    def resizeEvent(self, event):
        """Handle resize to update display"""
        super().resizeEvent(event)
        if self.original_pixmap:
            self.update_display()

    def reset_zoom(self):
        """Reset zoom to 100%"""
        self.zoom_factor = 1.0
        self.current_offset = QPoint(0, 0)
        self.update_display()

    def zoom_in(self):
        """Zoom in"""
        self.zoom_factor = min(self.max_zoom, self.zoom_factor + self.zoom_step)
        self.update_display()

    def zoom_out(self):
        """Zoom out"""
        self.zoom_factor = max(self.min_zoom, self.zoom_factor - self.zoom_step)
        self.update_display()

    def fit_to_window(self):
        """Fit image to window"""
        self.zoom_factor = 1.0
        self.current_offset = QPoint(0, 0)
        self.update_display()


class CardPreviewTab(QWidget):
    """A single tab for card preview with zoom controls"""

    # Forward signals from image label
    illustration_pan_changed = Signal(int, int)
    illustration_scale_changed = Signal(float)

    def __init__(self, title):
        super().__init__()
        self.title = title

        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        # Create scroll area for the image
        scroll_area = QScrollArea()
        scroll_area.setWidgetResizable(True)
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        # Create zoomable image label
        self.image_label = ZoomableImageLabel()
        self.image_label.setMinimumSize(400, 560)

        # Forward illustration mode signals
        self.image_label.illustration_pan_changed.connect(self.illustration_pan_changed)
        self.image_label.illustration_scale_changed.connect(self.illustration_scale_changed)

        scroll_area.setWidget(self.image_label)
        layout.addWidget(scroll_area)

        # Zoom controls
        from PySide6.QtWidgets import QHBoxLayout, QPushButton, QLabel as QLabeln
        controls_layout = QHBoxLayout()

        zoom_out_btn = QPushButton("-")
        zoom_out_btn.setMaximumWidth(30)
        zoom_out_btn.clicked.connect(self.image_label.zoom_out)
        controls_layout.addWidget(zoom_out_btn)

        zoom_reset_btn = QPushButton("100%")
        zoom_reset_btn.setMaximumWidth(50)
        zoom_reset_btn.clicked.connect(self.image_label.reset_zoom)
        controls_layout.addWidget(zoom_reset_btn)

        zoom_in_btn = QPushButton("+")
        zoom_in_btn.setMaximumWidth(30)
        zoom_in_btn.clicked.connect(self.image_label.zoom_in)
        controls_layout.addWidget(zoom_in_btn)

        controls_layout.addStretch()

        self.zoom_label = QLabeln("100%")
        controls_layout.addWidget(self.zoom_label)

        layout.addLayout(controls_layout)

        self.setLayout(layout)

    def set_illustration_mode(self, enabled):
        """Enable or disable illustration positioning mode"""
        self.image_label.set_illustration_mode(enabled)

    def set_image(self, image_buffer):
        """Set the image from a BytesIO buffer"""
        if not image_buffer:
            return

        data = image_buffer.read()
        image = QImage.fromData(data)
        pixmap = QPixmap.fromImage(image)
        self.image_label.setPixmap(pixmap)
        self.update_zoom_label()

    def update_zoom_label(self):
        """Update the zoom percentage label"""
        zoom_percent = int(self.image_label.zoom_factor * 100)
        self.zoom_label.setText(f"{zoom_percent}%")


class ImprovedCardPreview(QWidget):
    """Improved card preview with tabs for front/back and zoom capabilities"""

    # Signals for illustration mode changes (side, delta_x, delta_y)
    illustration_pan_changed = Signal(str, int, int)
    illustration_scale_changed = Signal(str, float)

    def __init__(self):
        super().__init__()

        self.illustration_mode_side = None  # Track which side is in illustration mode

        layout = QVBoxLayout()
        layout.setContentsMargins(5, 5, 5, 5)

        # Create tab widget
        self.tabs = QTabWidget()

        # Create front and back tabs
        self.front_tab = CardPreviewTab("Front")
        self.back_tab = CardPreviewTab("Back")

        # Connect illustration mode signals
        self.front_tab.illustration_pan_changed.connect(
            lambda dx, dy: self.illustration_pan_changed.emit('front', dx, dy)
        )
        self.front_tab.illustration_scale_changed.connect(
            lambda d: self.illustration_scale_changed.emit('front', d)
        )
        self.back_tab.illustration_pan_changed.connect(
            lambda dx, dy: self.illustration_pan_changed.emit('back', dx, dy)
        )
        self.back_tab.illustration_scale_changed.connect(
            lambda d: self.illustration_scale_changed.emit('back', d)
        )

        self.tabs.addTab(self.front_tab, tr("TAB_FRONT"))
        self.tabs.addTab(self.back_tab, tr("TAB_BACK"))

        layout.addWidget(self.tabs)

        self.setLayout(layout)

    def set_illustration_mode(self, enabled, side='front'):
        """Enable or disable illustration positioning mode for a specific side"""
        # Disable on both first
        self.front_tab.set_illustration_mode(False)
        self.back_tab.set_illustration_mode(False)

        if enabled:
            self.illustration_mode_side = side
            if side == 'front':
                self.front_tab.set_illustration_mode(True)
                self.tabs.setCurrentIndex(0)
            else:
                self.back_tab.set_illustration_mode(True)
                self.tabs.setCurrentIndex(1)
        else:
            self.illustration_mode_side = None

    def get_illustration_mode(self):
        return self.illustration_mode_side

    def set_card_images(self, front_buffer, back_buffer):
        """Set the front and back card images from BytesIO buffers"""
        if front_buffer:
            self.front_tab.set_image(front_buffer)

        if back_buffer:
            self.back_tab.set_image(back_buffer)

    def show_front(self):
        """Switch to front tab"""
        self.tabs.setCurrentIndex(0)

    def show_back(self):
        """Switch to back tab"""
        self.tabs.setCurrentIndex(1)



from watchdog.events import FileSystemEventHandler
import time

class CardFileHandler(FileSystemEventHandler):
    def __init__(self, card_renderer):
        super().__init__()
        self.card_renderer = card_renderer
        self.last_modified = 0

    def on_modified(self, event):
        if not event.is_directory and event.src_path == self.card_renderer.file_path:
            # Add a small delay to avoid multiple reload triggers
            current_time = time.time()
            if current_time - self.last_modified > 0.5:
                self.last_modified = current_time
                self.card_renderer.load_and_render_card()



from pathlib import Path
import pymupdf
from io import BytesIO
import subprocess
from subprocess import PIPE
from shoggoth import files


class Guide:
    def __init__(self, data, project, prince_cmd=None, prince_dir=None):
        self.path = data['path']
        self.id = data['id']
        self.data = data
        self.project = project
        self._html = None
        self._prince_cmd = prince_cmd
        self._prince_dir = prince_dir

    @property
    def prince_dir(self):
        if self._prince_dir is None:
            import shoggoth
            return shoggoth.app.config.get('Shoggoth', 'prince_dir') or None
        return self._prince_dir

    @property
    def prince_cmd(self):
        if self._prince_cmd is None:
            import shoggoth
            return shoggoth.app.config.get('Shoggoth', 'prince_cmd')
        return self._prince_cmd

    @property
    def target_path(self):
        return Path(self.path).parent / 'guide.pdf'

    def get_html(self):
        if not self._html:
            with open(self.path, 'r') as file:
                self._html = file.read()
        return self._html

    @property
    def front_page(self):
        return self.data.get('front_page', '')

    @front_page.setter
    def front_page(self, value):
        self.data['front_page'] = value

    @property
    def name(self):
        return self.data.get('name', 'Unnamed Guide')

    @name.setter
    def name(self, value):
        self.data['name'] = value

    def html_format(self, html) -> str:
        """ Does a simple string replacement for certain defined elements """
        html = html.replace("{{frontpage}}", self.front_page)
        html = html.replace("{{a4_empty}}", str(files.guide_dir/'guide_a4_empty.webp'))
        html = html.replace("{{a4_title}}", str(files.guide_dir/'guide_a4_title.webp'))
        html = html.replace("{{resolution_glyph_top}}", str(files.guide_dir/'resolution_glyph_top.png'))
        html = html.replace("{{resolution_glyph_bottom}}", str(files.guide_dir/'resolution_glyph_bottom.png'))
        return html

    def get_page(self, page, html: str = ''):
        if not html:
            p = subprocess.call([self.prince_cmd, self.path, '-o', str(self.target_path)], cwd=self.prince_dir)
            pdf = pymupdf.open(self.target_path)
        else:
            p = subprocess.run(
                [self.prince_cmd, '-', '-o', '-'],
                cwd=self.prince_dir,
                input=self.html_format(html).encode(),
                stdout=subprocess.PIPE,
            )
            data = p.stdout
            pdf = pymupdf.open(stream=data)

        image = pdf[page].get_pixmap().pil_image()

        buffer = BytesIO()
        image.save(buffer, format='jpeg', quality=90)
        buffer.seek(0)
        return buffer

    def save(self, html):
        self._html = html
        with open(self.path, 'w') as file:
            file.write(html)

    def render_to_file(self, html=None):
        if not html:
            html = self.get_html()
        html = self.html_format(html)

        p = subprocess.run(
            [self.prince_cmd, '-', '-o', str(self.target_path)],
            cwd=self.prince_dir,
            input=html.encode(),
        )
        return
